//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue;

import figue.event.PointingListenerInterface;
import figue.event.PointingMotionListenerInterface;
import figue.event.KeyListenerInterface;
import figue.event.HyperlinkListenerInterface;
import figue.event.PointingEvent;
import figue.event.EventMulticaster;
import figue.path.PathInterface;
import figue.resource.WrongResourceException;
import figue.geometry.Rectangle;
import figue.error.ImplementationError;
import figue.multithread.StoppedFacadeException;
import figue.box.GlyphInterface;
import figue.internal.AvailableVersion;
import figue.internal.Globals;
import figue.internal.InteractiveClientInterface;
import figue.internal.PairGlyphPath;
import figue.internal.PathCollectAreaVisitor;

import java.awt.Graphics;
import java.awt.Component;
import java.io.InputStream;
import antlr.ParserException;
import java.io.IOException;

/**
 * [PENDING: update the comments] InteractiveFacade.
 */

public abstract class InteractiveFacade extends Facade {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/InteractiveFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:32 $ Copyright 1998 INRIA.";

    protected InteractiveFacade(FacadeInitializer anInitializer) {
	super(AvailableVersion.GRAPHICAL.clone(null),anInitializer);
	internalInit(anInitializer);
    }

    protected InteractiveFacade(FacadeInitializer anInitializer,
				InputStream anInput) throws ParserException, IOException {
	super(AvailableVersion.GRAPHICAL.clone(null),anInitializer,anInput);
	internalInit(anInitializer);
    }

    protected InteractiveFacade(FacadeInitializer anInitializer,
				FacadeInterface aFacade) throws WrongResourceException, CloneNotSupportedException {
	super(AvailableVersion.GRAPHICAL.clone(null),anInitializer,aFacade);
	internalInit(anInitializer);
    }

    private final void internalInit(FacadeInitializer anInitializer) {
	if ( anInitializer.getScrollbarThickness() > 0 ) {
	    _scrollbarThickness = anInitializer.getScrollbarThickness();
	}
	else {
	    _scrollbarThickness = Globals.SCROLLBAR_THICKNESS;
	}
	if ( anInitializer.getHorizontalScrollbarPolicy() != null ) {
	    _horizontalScrollbarPolicy = anInitializer.getHorizontalScrollbarPolicy();
	}
	else {
	    _horizontalScrollbarPolicy = ScrollbarPolicy.AS_NEEDED;
	}
	if ( anInitializer.getVerticalScrollbarPolicy() != null ) {
	    _verticalScrollbarPolicy = anInitializer.getVerticalScrollbarPolicy();
	}
	else {
	    _verticalScrollbarPolicy = ScrollbarPolicy.AS_NEEDED;
	}
	if ( anInitializer.getAutoScrollPolicy() != null ) {
	    _autoScrollPolicy = anInitializer.getAutoScrollPolicy();
	}
	else {
	    _autoScrollPolicy = AutoScrollPolicy.WITHIN_FRAME;
	}
	if ( anInitializer.getInsureVisiblePolicy() != null ) {
	    _insureVisiblePolicy = anInitializer.getInsureVisiblePolicy();
	}
	else {
	    _insureVisiblePolicy = new InsureVisiblePolicy();
	}
	_rectangleCacheForPoint = new Rectangle();
	_rectangleCacheForBoundingBox = new Rectangle();
	_rectangleCacheForRefreshZone = new Rectangle();
	_keyListeners = null;
	_hyperlinkListeners = null;
	_pointingListeners = null;
	_pointingMotionListeners = null;
	_pathCollectAreaVisitor = new PathCollectAreaVisitor(this);
	switch ( _autoScrollPolicy.getEnum() ) {
	case AutoScrollPolicy.NONE_ENUM: {
	    break;
	}
	case AutoScrollPolicy.WITHIN_FRAME_ENUM: {
	    addPointingMotionListener(new PointingMotionListenerInterface() {
		public final void moving(PointingEvent anEvent) {
		    ;
		}
		public final void startDragging(PointingEvent anEvent) {
		    getClient().beginDrag(anEvent);
		}
		public final void dragging(PointingEvent anEvent) {
		    getClient().drag(anEvent);
		}
		public final void stopDragging(PointingEvent anEvent) {
		    getClient().endDrag(anEvent);
		}
	    });
	    break;
	}
	default: {
	    reportError(new ImplementationError("Unexpected"));
	    break;
	}
	}
	if ( anInitializer.getAutoScrollRate() != -1 ) {
	    _autoScrollRate = anInitializer.getAutoScrollRate();
	}
	else {
	    _autoScrollRate = Globals.AUTO_SCROLL_RATE;
	}
	if ( anInitializer.getAutoScrollDelay() != -1 ) {
	    _autoScrollDelay = anInitializer.getAutoScrollDelay();
	}
	else {
	    _autoScrollDelay = Globals.AUTO_SCROLL_DELAY;
	}
	if ( anInitializer.getAutoScrollMargin() != -1 ) {
	    _autoScrollMargin = anInitializer.getAutoScrollMargin();
	}
	else {
	    _autoScrollMargin = Globals.AUTO_SCROLL_MARGIN;
	}
    }

    protected final void init() {
	super.init();
	_pathCollectAreaVisitor.init();
	final java.lang.Thread theInitThread 
	    = new java.lang.Thread(new Runnable() {
		public final void run() {
		    getContext().init();
		}
	    });
	theInitThread.setPriority(java.lang.Thread.MIN_PRIORITY);
	theInitThread.start();
    }
    /**
     * [PENDING: update the comments] getClient
     * @level advanced
     */
    public abstract InteractiveClientInterface getClient();

    public abstract Component getAwtComponent();

    public final void update(Graphics aGraphic) {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\n-> update awt \n");
	}
	try {
	    synchronized ( getDrawable() ) {
		while ( true ) {
		    synchronized ( this ) {
			if ( getThreadState().isDrawable() ) {
			    getContext().setAwtContext(aGraphic);
			    getThreadState().startDraw();
			    executeDrawTask();
			    break;
			}
		    }
		    if ( getThreadState().isBuildingPhase() ) {
			return; /// TEMPO BUG FIX SWING
		    }
		    else {
			getDrawable().await();
		    }
		}
	    }
	    endDraw();
	}
	catch ( StoppedFacadeException anException ) {
	    return;
	}
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\n<- update awt\n");
	}
    }
    
    protected final void executeDrawTask() {
	final double theTime = System.currentTimeMillis();
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("-> executeDrawTask");
	}
	GlyphInterface theRoot = getRootGlyph();
	Rectangle theClip = getContext().getClipBounds();
	getContext().setRefreshZone(theClip.getX(),theClip.getY(),theClip.getWidth(),theClip.getHeight());
	_rectangleCacheForRefreshZone.setBounds(getContext().getRefreshZone());
	_rectangleCacheForBoundingBox.setBounds(0,0,0,0);
	getVersion().draw(theRoot,null,getSelectionPath(),
			  _rectangleCacheForRefreshZone,_rectangleCacheForBoundingBox);
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("<- executeDrawTask " + (System.currentTimeMillis() - theTime));
	}
    }

    protected final PairGlyphPath getDeepestContains(int x, int y) {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("-> getDeepestContains");
	}
	try {
	    synchronized ( getInactive() ) {
		while ( true ) {
		    synchronized ( this ) {
			if ( getThreadState().isBuildingPhase() ) {
			    return null;
			}
			else {
			    if ( getThreadState().isInactive() ) {
				final PairGlyphPath theResult = getClient().getDeepestContains(x,y,getRootGlyph(),_rectangleCacheForPoint,getContext());
				if ( Globals.DEBUG_MULTI_THREAD ) {
				    System.out.println("<- getDeepestContains");
				}
				return theResult;
			    }
			}
		    }
		    getInactive().await();
		}
	    }
	}
	catch ( StoppedFacadeException anException ) {
	    return null;
	}
    }

    protected final void insureVisibleWhenInactive(PathInterface aPath) {
	_pathCollectAreaVisitor.reinit();
	aPath.accept(_pathCollectAreaVisitor,getRootGlyph());
	getClient().insureVisible(_pathCollectAreaVisitor.getZone());
    }

    public final void addKeyListener(KeyListenerInterface aListener) {
	_keyListeners = EventMulticaster.addKeyListener(_keyListeners,aListener);
    }
    
    public final void removeKeyListener(KeyListenerInterface aListener) {
	_keyListeners = EventMulticaster.removeKeyListener(_keyListeners,aListener);
    }
    
    public final void addPointingListener(PointingListenerInterface aListener) {
	_pointingListeners = EventMulticaster.addPointingListener(_pointingListeners,aListener);
    }
    
    public final void removePointingListener(PointingListenerInterface aListener) {
	_pointingListeners = EventMulticaster.removePointingListener(_pointingListeners,aListener);
    }
    
    public final void addPointingMotionListener(PointingMotionListenerInterface aListener) {
	_pointingMotionListeners = EventMulticaster.addPointingMotionListener(_pointingMotionListeners,aListener);
    }
    
    public final void removePointingMotionListener(PointingMotionListenerInterface aListener) {
	_pointingMotionListeners = EventMulticaster.removePointingMotionListener(_pointingMotionListeners,aListener);
    }
    
    public final void addHyperlinkListener(HyperlinkListenerInterface aListener) {
	_hyperlinkListeners = EventMulticaster.addHyperlinkListener(_hyperlinkListeners,aListener);
    }
    
    public final void removeHyperlinkListener(HyperlinkListenerInterface aListener) {
	_hyperlinkListeners = EventMulticaster.removeHyperlinkListener(_hyperlinkListeners,aListener);
    }

    protected final KeyListenerInterface getKeyListeners() {
	return _keyListeners;
    }
    
    public final HyperlinkListenerInterface getHyperlinkListeners() {
	return _hyperlinkListeners;
    }
    
    protected final PointingListenerInterface getPointingListeners() {
	return _pointingListeners;
    }
    
    protected final PointingMotionListenerInterface getPointingMotionListeners() {
	return _pointingMotionListeners;
    }

    final int getScrollbarThickness() {
	return _scrollbarThickness;
    }
    
    final ScrollbarPolicy getHorizontalScrollbarPolicy() {
	return _horizontalScrollbarPolicy;
    }
    
    final ScrollbarPolicy getVerticalScrollbarPolicy() {
	return _verticalScrollbarPolicy;
    }
    
    final InsureVisiblePolicy getInsureVisiblePolicy() {
	return _insureVisiblePolicy;
    }
    
    final int getAutoScrollRate() {
	return _autoScrollRate;
    }
    
    final int getAutoScrollDelay() {
	return _autoScrollDelay;
    }
    
    final int getAutoScrollMargin() {
	return _autoScrollMargin;
    }

    public final void refresh(Rectangle aRectangle) {
	getClient().refresh(aRectangle);
    }
    
    public final void refresh() {
	getClient().refresh();
    }
    
    public final void requestFocus() {
	getClient().requestFocus(); 
    }
    
    protected void dispose() {
	_rectangleCacheForBoundingBox = null;
	_rectangleCacheForRefreshZone = null;
	_rectangleCacheForPoint = null;
	_keyListeners = null;
	_hyperlinkListeners = null;
	_pointingListeners = null;
	_pointingMotionListeners = null;
	_pathCollectAreaVisitor = null;
	_horizontalScrollbarPolicy = null;
	_verticalScrollbarPolicy = null;
	_autoScrollPolicy = null;
	super.dispose();
    }

    private Rectangle _rectangleCacheForBoundingBox;
    private Rectangle _rectangleCacheForRefreshZone;
    private Rectangle _rectangleCacheForPoint;
    private KeyListenerInterface _keyListeners;
    private HyperlinkListenerInterface _hyperlinkListeners;
    private PointingMotionListenerInterface _pointingMotionListeners;
    private PointingListenerInterface _pointingListeners;
    private PathCollectAreaVisitor _pathCollectAreaVisitor;
    private int _scrollbarThickness;
    private int _autoScrollRate;
    private int _autoScrollDelay;
    private int _autoScrollMargin;
    private ScrollbarPolicy _horizontalScrollbarPolicy;
    private ScrollbarPolicy _verticalScrollbarPolicy;
    private AutoScrollPolicy _autoScrollPolicy;
    private InsureVisiblePolicy _insureVisiblePolicy;
    
}

