//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.error.ImplementationError;
import figue.internal.Globals;

/**
 * [PENDING: update the comments] CompositeGlyphArity1.
 * @level advanced
 */

public abstract class CompositeGlyphArity1 extends CompositeGlyph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/CompositeGlyphArity1.java,v $ $revision$ $Date: 1998/11/02 15:54:37 $ Copyright 1998 INRIA.";
  
    public CompositeGlyphArity1() { 
	_child = null; 
    }

    public Object clone() throws CloneNotSupportedException {
 	final CompositeGlyphArity1 theResult = (CompositeGlyphArity1) super.clone();
	if ( _child != null ) {
	    theResult.addChild((GlyphInterface)_child.clone());
	}
	else {
	    theResult._child = null;
	}
 	return theResult;
    }
  
    public final GlyphInterface getChild(int aRank) {
	if ( Globals.ROBUST ) {
	    if ( (aRank != 0) || (_child == null) ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	return _child;
    }
  
    public final void insertChild(int aRank, GlyphInterface aGlyph) {
	super.insertChild(aRank,aGlyph);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = aGlyph;
    }
  
    public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	super.replaceChild(aRank,aGlyph);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = aGlyph;
    }
  
    public final void removeChild(int aRank) {
	super.removeChild(aRank);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = null;
    }
  
    public final int numberOfChildren() { 
	if ( _child != null ) {
	    return 1;
	}
	return 0;
    }

    protected final SelectionPathInterface downInSelection(SelectionPathInterface aSelectionPath) {
	SelectionPathInterface theSon = null;
	final int theSize = aSelectionPath.numberOfChildren();
	if ( theSize > 0 ) {
	    if ( Globals.ROBUST ) {
		if ( theSize != 1 ) {
		    throw new ImplementationError(
						  "To many children for the selection: " +
						  toString() + ": " +
						  aSelectionPath.toString());
		}
	    }
	    theSon = aSelectionPath.getChild(theSize - 1);
	    if ( Globals.ROBUST ) {
		if ( theSon.getRank() != 0 ) {
		    throw new ImplementationError(
						  "Wrong logical rank for the selection: " +
						  toString() + ": " +
						  theSon.toString());
		}
	    }
	}
	return theSon;
    }
  
    private GlyphInterface _child; 

}
