//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.path.SelectionPathInterface;
import figue.resource.FontMetricInterface;
import figue.selection.HighlighterInterface;
import figue.selection.DefaultHighlighter;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.ModificationInterface;
import figue.format.ModificationStatus;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyComputeSizeTill;
import figue.format.StopFormattingSonsException;
import figue.format.ComputedValueInterface;
import figue.format.ActionNewLine;
import figue.error.ErrorHandlerInterface;
import figue.error.DefaultErrorHandler;
import figue.error.Warning;
import figue.error.Error;
import figue.error.ShouldNotBeCalledError;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.BooleanWrapper;
import figue.internal.Direction;
import figue.internal.PairGlyphPath;
import figue.FacadeInterface;
import figue.InteractiveFacade;
import figue.internal.Cursor;

import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.Writer;

/**
 * [PENDING: update the comments] DecoratedGlyphAbstract.
 * @level advanced
 */

public abstract class DecoratedGlyphAbstract extends GlyphInterface implements ErrorHandlerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/DecoratedGlyphAbstract.java,v $ $revision$ $Date: 1998/11/02 15:54:37 $ Copyright 1998 INRIA.";
  
    protected DecoratedGlyphAbstract() {
	super();
	_parent = null;
    }

    protected abstract GlyphInterface getComponent();
    protected abstract void setComponent(GlyphInterface aGlyph);

    public final GlyphInterface getParent() {
	return _parent;
    }

    protected final void setParent(GlyphInterface aGlyph) {
	_parent = aGlyph;
    }

    protected final void clearParent() {
	_parent = null;
    }
  
    public GlyphInterface getChild(int aRank) {
	return getComponent().getChild(aRank);
    }

    public final void addChild(GlyphInterface aGlyph) {
	insertChild(numberOfChildren(),aGlyph);
    }

    public void insertChild(int aRank, GlyphInterface aGlyph) {
	getComponent().insertChild(aRank,aGlyph);
    }

    public void replaceChild(int aRank, GlyphInterface aGlyph) {
	getComponent().replaceChild(aRank,aGlyph);
    }

    public final void removeChild() {
	removeChild(numberOfChildren() - 1);
    }

    public void removeChild(int aRank) {
	getComponent().removeChild(aRank);
    }
  
    public int indexOf(GlyphInterface aGlyph) {
	if ( getComponent() == aGlyph ) {
	    return 0;
	}
	else {
	    return -1;
	}
    }

    protected PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	final PathInterface theResult = new LinearPath(indexOf(aGlyph));
	if ( aPath != null ) {
	    theResult.addChild(aPath);
	}
	return getParent().computePathTo(this,theResult);
    }

    public int numberOfChildren() {
	return getComponent().numberOfChildren();
    }

    public final void notifyChange() {
	if ( _parent != null ) {
	    _parent.notifyChange();
	}
    }
  
    public final void notifyEagerChange() {
	if ( _parent != null ) {
	    _parent.notifyEagerChange();
	}
    }

    public final ComputedValueInterface getDependent() {
	return getComponent().getDependent();
    }
  
    public final void addDependent(ComputedValueInterface aValue) {
	getComponent().addDependent(aValue);
    }
  
    public final void removeDependent() {
	getComponent().removeDependent();
    }

    public final void reportError(Error anError) {
	if ( canHandleError() ) {
	    handleError(anError);
	}
	else {
	    if ( hasNextErrorHandler() ) {
		getNextErrorHandler().reportError(anError);
	    }
	    else {
		if ( DefaultErrorHandler.DEFAULT != null ) {
		    DefaultErrorHandler.DEFAULT.reportError(anError);
		}
		else {
		    DefaultErrorHandler.INSTANCE.reportError(anError);
		}
	    }      
	}
    }

    public final void handleError(Error anError) {
	anError.managedBy(this);
    }

    public boolean hasNextErrorHandler() {
	return ( _parent != null );
    }
  
    public ErrorHandlerInterface getNextErrorHandler() {
	return _parent;
    }

    public final void manageFatalError(Error anError) {
	throw new ShouldNotBeCalledError();
    }

    public final void manageWarning(Warning aWarning) {
	throw new ShouldNotBeCalledError();
    }

    public final boolean canHandleError() {
	return false;
    }

    public int getOriginX() {
	return getComponent().getOriginX();
    }
  
    public int getOriginY() {
	return getComponent().getOriginY();
    }
  
    public void setOrigin(int x, int y) {
	getComponent().setOrigin(x,y);
    }
  
    public void resetOrigin(int x, int y, ContextInterface aContext) {
	getComponent().resetOrigin(x,y,aContext);
    }

    protected final Dimension getSize() {
	return getComponent().getSize();
    }
  
    protected final Dimension getEagerSize() {
	return getComponent().getEagerSize();
    }

    public Dimension getSizeValue() {
	return getComponent().getSizeValue();
    }
  
    public void setSize(Dimension aSize) {
	getComponent().setSize(aSize);
    }
  
    public void resetSize(Dimension aSize, ContextInterface aContext) {
	getComponent().resetSize(aSize,aContext);
    }

    public void update() {
	;
    }
  
    public void updateEager() {
	;
    }

    protected Dimension computeSize(ContextInterface aContext,
				    StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				  
				    StrategySetOrigin aSetOriginStrategy,
				    StrategyComputeSizeTill aComputeSizeTillStrategy) {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    public void format2D(ContextInterface aContext,
			 FontMetricInterface aFontMetric,
			 AlternativeInterface anAlternative,
			 DistanceInterface aDistance,
			 StrategyReadyToFormat aReadyToFormatStrategy,
			 StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			 StrategySetOrigin aSetOriginStrategy,
			 StrategySetSize aSetSizeStrategy,
			 boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	getComponent().format2D(aContext,aFontMetric,anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
	aDistance.checkAfter(this,anAlternative);
    }

    public boolean redispatchEvent(InteractiveFacade aFacade,int aMouseEventType,MouseEvent anEvent) {
	return getComponent().redispatchEvent(aFacade,aMouseEventType,anEvent);
    }

    public int computeDistance2D(DistanceInterface aDistance,
				 DecalageInterface aDecalage) {
	return getComponent().computeDistance2D(aDistance,aDecalage);
    }
  
    public int computeDecalage2D(GlyphInterface aGlyph,
				 DecalageInterface aDecalage) {
	return getComponent().computeDecalage2D(aGlyph,aDecalage);
    }

    public int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return getComponent().computeInitialDecalage2D(aDecalage);
    }

    public void tryAlternative(AlternativeInterface anAlternative) {
	getComponent().tryAlternative(anAlternative);
    }
  
    public void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	getComponent().tryAlternativeForInitial(anAlternative);
    }
  
    public void tryAlternativeForDistance(DistanceInterface aDistance,
					  AlternativeInterface anAlternative) {
	getComponent().tryAlternativeForDistance(aDistance,anAlternative);
    }
  
    public void tryAlternativeForDecalage(DecalageInterface aDecalage,
					  AlternativeInterface anAlternative) {
	getComponent().tryAlternativeForDecalage(aDecalage,anAlternative);
    }
  
    public boolean isClean() {
	return getComponent().isClean();
    }
  
    public boolean isDirty() {
	return getComponent().isDirty();
    }

    public boolean isNew() {
	return getComponent().isNew();
    }
  
    public boolean isProceed() {
	return getComponent().isProceed();
    }
  
    public boolean isUnstable() {
	return getComponent().isUnstable();
    }
  
    public void setClean() {
	getComponent().setClean();
    }
  
    public void setProceed() {
	getComponent().setProceed();
    }
  
    public void setDirty() {
	getComponent().setDirty();
    }

    public void setNew() {
	getComponent().setNew();
    }
  
    public void setUnstable() {
	getComponent().setUnstable();
    }

    public final void markAllBranchDirty(SonModificationStatus aStatus,
					 ContextInterface aContext) {
	GlyphInterface theCurrentGlyph = this;
	GlyphInterface theSon;
	SonModificationStatus theStatus = aStatus;
	boolean theEndFlag = true;
	while ( theEndFlag ) {
	    if ( theCurrentGlyph.isClean() ) {
		theCurrentGlyph.setDirty();
		if ( ! theCurrentGlyph.getModification().isModified() ) {
		    theCurrentGlyph.setModification(aContext.newModification(ModificationStatus.SON));
		}
	    }
	    theSon = theCurrentGlyph;
	    theCurrentGlyph = theCurrentGlyph.getParent();
	    theEndFlag = ( ( theCurrentGlyph != null ) &&
			   theCurrentGlyph.sonModified(theSon,theStatus,aContext) );
	    if ( theStatus == SonModificationStatus.KILL ) {
		theStatus = SonModificationStatus.SHRINK;
	    }
	}
    }

    public void cleanModificationAndUpdateRefreshZone(Rectangle aVisibleZone,
						      Rectangle aRefreshZone,
						      BooleanWrapper aRefreshZoneNotInitializedFlag,
						      Rectangle aRectangle,
						      ContextInterface aContext) {
	getComponent().cleanModificationAndUpdateRefreshZone(aVisibleZone,aRefreshZone,aRefreshZoneNotInitializedFlag,aRectangle,aContext);
    }

    protected  void cleanModification(ContextInterface aContext) {
	getComponent().cleanModification(aContext);
    }

    public PairGlyphPath getDeepestContains(int x,
					    int y, 
					    Rectangle aBoundingBox,
					    ContextInterface aContext) {
    
	return getComponent().getDeepestContains(x,y,aBoundingBox,aContext);
    }

    public Cursor getCursor() {
	return getComponent().getCursor();
    }
    
    protected void setCursor(Cursor aCursor) {
	getComponent().setCursor(aCursor);
    }
  
    protected boolean isFinalTarget() {
	return getComponent().isFinalTarget();
    }

    protected void refresh() {
	getComponent().refresh();
    }

    public boolean draw2D(ContextInterface aContext,
			  Transformer aTransformer,
			  SelectionPathInterface aSelectionPath,
			  Rectangle aRefreshZone,
			  Rectangle aBoundingBox) {
	return getComponent().draw2D(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       SelectionPathInterface aSelectionPath,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	getComponent().draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
    }

    protected boolean draw2D(ContextInterface aContext,
			     Transformer aTransformer,
			     Rectangle aRefreshZone,
			     Rectangle aBoundingBox) {
	return getComponent().draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	getComponent().draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    public HighlighterInterface getHighlighter(SelectionPathInterface aSelectionPath) {
	return DefaultHighlighter.INSTANCE;
    }

    protected ContextMementoInterface draw2DSelections(ContextInterface aContext,
						       Transformer aTransformer,
						       SelectionPathInterface aSelectionPath) {
	return getComponent().draw2DSelections(aContext,aTransformer,aSelectionPath);
    }

    protected SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
						     int aRank) {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    public FacadeInterface getFacade() {
	return _parent.getFacade();
    }
    
    public boolean improveEstimatedHeightAndCheckAwake(int anHeight) {
	return _parent.improveEstimatedHeightAndCheckAwake(anHeight);
    }

    public void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
						      int aRank) {
	_parent.improveSizeEstimationAndUpdateParents(aContext,
						      _parent.indexOf(this));
    }

    protected final StrategyImproveSizeEstimation chooseAlgorithm(StrategyImproveSizeEstimation aStrategy) {
	return getComponent().chooseAlgorithm(aStrategy);
    }
    
    protected void setSizeEstimation(Dimension aCurrentSize) {
	getComponent().setSizeEstimation(aCurrentSize);
    }
  
    public Dimension getSizeEstimation() {
	return getComponent().getSizeEstimation();
    }
    
    protected void computeBoundingBox(Rectangle aRectangle,
				      Transformer aTransformer) {
	getComponent().computeBoundingBox(aRectangle,aTransformer);
    }
  
    public void computeBoundingBox(Rectangle aRectangle) {
	getComponent().computeBoundingBox(aRectangle);
    }
    
    public void computeOldBoundingBox(Rectangle aRectangle,
				      int anOldX,
				      int anOldY,
				      Dimension anOldSize) {
	getComponent().computeOldBoundingBox(aRectangle,anOldX,anOldY,anOldSize);
    }
    
    protected void computeEstimatedBoundingBox(Rectangle aRectangle,
					       Transformer aTransformer) {
	getComponent().computeEstimatedBoundingBox(aRectangle,aTransformer);
    }
  
    protected void computeEstimatedBoundingBox(Rectangle aRectangle) {
	getComponent().computeEstimatedBoundingBox(aRectangle);
    }
    
    protected Direction checkRefreshZone(Rectangle aRefreshZone,
					 Rectangle aBoundingBox) {
	return getComponent().checkRefreshZone(aRefreshZone,aBoundingBox);
    }
    
    protected  Direction checkRefreshZoneAgainstEstimation(Rectangle aRefreshZone,
							   Rectangle aBoundingBox) {
	return getComponent().checkRefreshZoneAgainstEstimation(aRefreshZone,aBoundingBox);
    }

    protected void doTranslate(Rectangle aRectangle) {
	getComponent().doTranslate(aRectangle);
    }
  
    protected void undoTranslate(Rectangle aRectangle) {
    }

    public void doTranslatesZones(Rectangle aRefreshZone,
				  Rectangle aVisibleZone,
				  int aRank) {
	getComponent().doTranslatesZones(aRefreshZone,aVisibleZone,aRank);
    }
    public void undoTranslateZones(Rectangle aRefreshZone,
				      Rectangle aVisibleZone,
				   int aRank) {
	getComponent().undoTranslateZones(aRefreshZone,aVisibleZone,aRank);
    }

    public boolean isActionNoneComplient() {
	return getComponent().isActionNoneComplient();
    }
  
    public boolean isActionNewLineComplient(ActionNewLine anAction) {
	return getComponent().isActionNewLineComplient(anAction);
    }
  
    public void executeActionNewLine(GlyphInterface aGlyph) 
	throws StopFormattingSonsException {
	getComponent().executeActionNewLine(aGlyph);
    }
  
    public boolean isActionShrinkComplient() {
	return getComponent().isActionShrinkComplient();
    }

    public void executeActionShrink() throws StopFormattingSonsException {
	getComponent().executeActionShrink();
    }

    public boolean isActionShrinkSonComplient() {
	return getComponent().isActionShrinkSonComplient();
    }

    public void executeActionShrinkSon() {
	getComponent().executeActionShrinkSon();
    }

    protected ModificationInterface getModification() {
	return getComponent().getModification();
    }
  
    protected void setModification(ModificationInterface aModification) {
	getComponent().setModification(aModification);
    }

    protected boolean sonModified(GlyphInterface aGlyph,
				  SonModificationStatus aStatus,
				  ContextInterface aContext) {
	return true;
    }

    public void computeCuttingCosts(AlternativeInterface anAlternative,
				    Rectangle aRectangle,
				    Transformer aTransformer) {
	getComponent().computeCuttingCosts(anAlternative,aRectangle,aTransformer);
    }
  
    public String toString() {
	return getComponent().toString();
    }

    public final void writeAsXML(Writer anOutput) throws IOException {
	anOutput.write("<?XML version=\"1.0\"?>");
	writeState(anOutput);
    }

    public void writeState(Writer anOutput) throws IOException {
	anOutput.write("<");
	anOutput.write(getName());
	writeIntrinsicState(anOutput);
	if ( writeExtrinsicState(anOutput) ) {
	    anOutput.write("</");
	    anOutput.write(getName());
	    anOutput.write(">");
	}
	else {
	    anOutput.write("/>");
	}
    }

    protected boolean writeExtrinsicState(Writer anOutput) throws IOException {
	anOutput.write('>');
	if ( getComponent() != NullGlyph.INSTANCE ) {
	    getComponent().writeState(anOutput);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected void writeIntrinsicState(Writer anOutput) throws IOException {
    }

    private GlyphInterface _parent;
  
}
