//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.resource.FontMetricInterface;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;

/**
 * [PENDING: update the comments] FlyWeightAtomInterface.
 * @level advanced
 */

public abstract class FlyWeightAtom implements FlyWeightAtomInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/FlyWeightAtom.java,v $ $revision$ $Date: 1998/11/02 15:54:38 $ Copyright 1998 INRIA.";
  
    protected FlyWeightAtom() {
	super();
	_size = null;
	_counter = 0;
	_fontMetric = null;
    }
  
    protected FlyWeightAtom(FontMetricInterface aFontMetric) {
	super();
	_size = null;
	_counter = 0;
	_fontMetric = aFontMetric;
    }

    public final FontMetricInterface getFontMetric() {
	return _fontMetric;
    }

    public final void setFontMetric(FontMetricInterface aFontMetric) {
	_fontMetric = aFontMetric;
    }

    public final Dimension getSize() {
	return _size;
    }
  
    public final void setSize(Dimension aSize) {
	_size = aSize;
    }

    public final void dispose() {
	//     decrReferenceCounter(); ///// TEMPO SHOULD DECREMENT
    }
  
    protected final void incrCounter() {
	_counter++;
    }

    protected final void incrReferenceCounter(ContextInterface aContext) {
	if ( _counter++ == 0 ) {
	    insertInRegistry(aContext);
	}
    }

    protected final void decrReferenceCounter(ContextInterface aContext) {
	if ( --_counter == 0 ) {
	    removeFromRegistry(aContext);
	}
    }

    protected abstract void insertInRegistry(ContextInterface aContext);
    protected abstract void removeFromRegistry(ContextInterface aContext);

    public final void draw2DClean(GlyphInterface anOwner,
				  ContextInterface aContext,
				  Transformer aTransformer,
				  SelectionPathInterface aSelectionPath) {
	ContextMementoInterface theMemento = anOwner.draw2DSelections(aContext,aTransformer,aSelectionPath);
	draw2DClean(anOwner,aContext,aTransformer);
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }
  
    public final Dimension getSizeValue() {
	return _size;
    }

    public final FlyWeightAtomInterface format2D(ContextInterface aContext,
						 FontMetricInterface aFontMetric,
						 DistanceInterface aDistance) {
	if ( ( _fontMetric != null ) && ( aFontMetric.equals(_fontMetric) ) ) {
	    return this;
	}
	FlyWeightAtomInterface theFlyWeight = updateRegistry(aContext,aFontMetric);
	final FontMetricInterface theFontMetric = theFlyWeight.getFontMetric();
	if ( theFlyWeight.getSize() == null ) {
	    theFlyWeight.setSize(new Dimension(theFontMetric.getAscent(),
					       theFlyWeight.computeWidth(theFontMetric),
					       theFontMetric.getDescent()));
	}
	return theFlyWeight;
    }

    protected abstract FlyWeightAtomInterface updateRegistry(ContextInterface aContext,
							     FontMetricInterface aFontMetric);
    
    private Dimension _size;
    private int _counter;
    private FontMetricInterface _fontMetric;
  
}

