/**********8


package figue;

import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

public class HorizOrVerti extends DecoratedGlyphAbstract {

    public HorizOrVerti(MeasureInterface anInterWord,
			MeasureInterface anIndentation,
			MeasureInterface anInterline) {
	_interWord = anInterWord;
	_interline = anInterline;
	_indent = anIndentation;
	_children = new VectorGlyph(5,5);
	_brokenFlag = false;
	_h = new H(anInterWord);
	_v = new V(anIndentation,anInterline);
	_component = _h;
	_component.setParent(this);
    }

    public HorizOrVerti(int anInterWord, int anIndentation, int anInterline) {
	this(new Measure(anInterWord,UnitInterface.BLANK_TENTH),
	     new Measure(anIndentation,UnitInterface.BLANK_TENTH),
	     new Measure(anInterline,UnitInterface.BLANK_TENTH));
    }

    public HorizOrVerti() {
	this(10,0,0);
    }

    public final MeasureInterface getInterWord() {
	return _interWord;
    }

    public final void setInterWord(MeasureInterface anInterWord) {
	_interWord = anInterWord;
	_h.setInterWord(anInterWord);
    }

    public final void setInterWord(int anInterWord) {
	setInterWord(new Measure(anInterWord,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getInterline() {
	return _interline;
    }

    public final void setInterline(MeasureInterface anInterline) {
	_interline = anInterline;
	_v.setInterline(anInterline);
    }

    public final void setInterline(int anInterline) {
	setInterline(new Measure(anInterline,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getIndentation() {
	return _indent;
    }

    public final void setIndentation(MeasureInterface anIndentation) {
	_indent = anIndentation;
	_v.setIndentation(anIndentation);
    }

    public final void setIndentation(int anIndentation) {
	setIndentation(new Measure(anIndentation,UnitInterface.BLANK_TENTH));
    }

    protected final GlyphInterface getComponent() {
	return  _component;
    }

    protected final void setComponent(GlyphInterface aGlyph) {
	reportError(new ShouldNotBeCalledError());
    }

    protected final void format2D(ContextInterface aContext,
				  FontMetricInterface aFontMetric,
				  AlternativeInterface anAlternative,
				  DistanceInterface aDistance,
				  StrategyReadyToFormat aReadyToFormatStrategy,
				  StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				
				  StrategySetOrigin aSetOriginStrategy,
				  StrategySetSize aSetSizeStrategy,
				  boolean aMustFormatFlag) 
	throws StopFormattingSonsException {
	if ( aMustFormatFlag || isNew() ) {
	    flat();
	}
	DistanceInterface theNewDistance = aDistance.lowerLevel(aContext,this);
	theNewDistance.checkBefore(anAlternative);
	getComponent().format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy.chooseAlgorithmForChildren(this),aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
	theNewDistance.newBrother(aContext,getComponent());
	theNewDistance.upperLevel(aContext);
	//	super.format2D(aContext,aFontMetric,anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
    }

    private final void flat() {
	if ( _brokenFlag ) {
	    _brokenFlag = false;
	    _component = _h;
	}
    }

    private final void breakLine() {
	_brokenFlag = true;
	_component = _v;
    }

    protected boolean isActionShrinkComplient() {
	System.out.println("HoV.isActionShrinkComplient");
	return ( ! _brokenFlag );
    }
    
    protected void executeActionShrink() throws StopFormattingSonsException {
	breakLine();
	removeDependent(); /// TEMPO TEMPO
	throw new StopFormattingSonsException(this);
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("HoV<");
	final int theSize = numberOfChildren();
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	   theResult.append(theRank).append(':').append(getChild(theRank));
	}
	theResult.append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "HOV";
    }

    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" Interword=\"");
	anOutput.write(Integer.toString(_interWord.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interWord.getUnit());
	anOutput.write(" Interline=\"");
	anOutput.write(Integer.toString(_interline.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interline.getUnit());
	anOutput.write(" Indent=\"");
	anOutput.write(Integer.toString(_indent.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_indent.getUnit());
    }
    
    private MeasureInterface _interWord;
    private MeasureInterface _interline;
    private MeasureInterface _indent;
    private VectorGlyph _children;
    private boolean _brokenFlag;
    private GlyphInterface _component;
    private H _h;
    private V _v;

    final class V extends VerticalAbstract {
	
	public V(MeasureInterface anIndentation,
		 MeasureInterface anInterline) {
	    super(anIndentation,anInterline);
	}
	
	public final GlyphInterface getChild(int aRank) {
	    return _children.elementAt(aRank);
	}
	
	public final void insertChild(int aRank, GlyphInterface aGlyph) {
	    super.insertChild(aRank,aGlyph);
	    _children.insertElementAt(aGlyph,aRank);
	}
	
	public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	    super.replaceChild(aRank,aGlyph);
	    _children.setElementAt(aGlyph,aRank);
	}
	
	public final void removeChild(int aRank) {
	    super.removeChild(aRank);
	    _children.removeElementAt(aRank);
	}
	
	public final int numberOfChildren() { 
	    return _children.size(); 
	}

    }

    final class H extends HorizontalAbstract {

	public H(MeasureInterface anInterWord) {
	    super(anInterWord);
	}

	public final GlyphInterface getChild(int aRank) {
	    return _children.elementAt(aRank);
	}
	
	public final void insertChild(int aRank, GlyphInterface aGlyph) {
	    super.insertChild(aRank,aGlyph);
	    _children.insertElementAt(aGlyph,aRank);
	}
	
	public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	    super.replaceChild(aRank,aGlyph);
	    _children.setElementAt(aGlyph,aRank);
	}
	
	public final void removeChild(int aRank) {
	    super.removeChild(aRank);
	    _children.removeElementAt(aRank);
	}
	
	public final int numberOfChildren() { 
	    return _children.size(); 
	}
    
    }
    
}


*******/
