//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.resource.FontMetricInterface;
import figue.geometry.PositionToFather;
import figue.geometry.PositionToBrother;
import figue.geometry.Point;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.SonModificationStatus;
import figue.format.ModificationInterface;
import figue.format.Modification;
import figue.format.ModificationStatus;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.error.ImplementationError;
import figue.internal.Globals;
import figue.internal.ContextInterface;
import figue.MeasureInterface;
import figue.UnitInterface;
import figue.Measure;

import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] Insets.
 */

public class Insets extends Combinator {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Insets.java,v $ $revision$ $Date: 1998/11/02 15:54:41 $ Copyright 1998 INRIA.";
    
    public Insets(MeasureInterface aSpaceBefore,
		  MeasureInterface aSpaceAfter,
		  MeasureInterface aLeftMargin,
		  MeasureInterface aRightMargin) {
	super();
	_spaceBefore = aSpaceBefore;
	_spaceAfter = aSpaceAfter;
	_leftMargin = aLeftMargin;
	_rightMargin = aRightMargin;
	_child = null;
    }

    public Insets(int aSpaceBefore,
		  int aSpaceAfter,
		  int aLeftMargin,
		  int aRightMargin) {
	this(new Measure(aSpaceBefore,UnitInterface.BLANK_TENTH),
	     new Measure(aSpaceAfter,UnitInterface.BLANK_TENTH),
	     new Measure(aLeftMargin,UnitInterface.BLANK_TENTH),
	     new Measure(aRightMargin,UnitInterface.BLANK_TENTH));
    }

    public Insets() {
	this(0,0,0,0);
    }

    public Object clone() throws CloneNotSupportedException {
 	final Insets theResult = (Insets) super.clone();
	theResult._spaceBefore = (MeasureInterface)_spaceBefore.clone();
	theResult._spaceAfter = (MeasureInterface)_spaceAfter.clone();
	theResult._leftMargin = (MeasureInterface)_leftMargin.clone();
	theResult._rightMargin = (MeasureInterface)_rightMargin.clone();
	if ( _child != null ) {
	    theResult.addChild((GlyphInterface)_child.clone());
	}
	else {
	   theResult._child = null;
	}
 	return theResult;
    }

    public final MeasureInterface getSpaceBefore() {
	return _spaceBefore;
    }

    public final void setSpaceBefore(MeasureInterface aSpaceBefore) {
	_spaceBefore = aSpaceBefore;
    }
  
    public final void setSpaceBefore(int aSpaceBefore) {
	setSpaceBefore(new Measure(aSpaceBefore,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getSpaceAfter() {
	return _spaceAfter;
    }
  
    public final void setSpaceAfter(MeasureInterface aSpaceAfter) {
	_spaceAfter = aSpaceAfter;
    }

    public final void setSpaceAfter(int aSpaceAfter) {
	setSpaceAfter(new Measure(aSpaceAfter,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getLeftMargin() {
	return _leftMargin;
    }

    public final void setLeftMargin(MeasureInterface aLeftMargin) {
	_leftMargin = aLeftMargin;
    }
  
    public final void setLeftMargin(int aLeftMargin) {
	setLeftMargin(new Measure(aLeftMargin,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getRightMargin() {
	return _rightMargin;
    }

    public final void setRightMargin(MeasureInterface aRightMargin) {
	_rightMargin = aRightMargin;
    }
  
    public final void setRightMargin(int aRightMargin) {
	setRightMargin(new Measure(aRightMargin,UnitInterface.BLANK_TENTH));
    }

    protected final void preFormat2D(ContextInterface aContext,
				     FontMetricInterface aFontMetric) {
	_spaceBefore.setFontMetric(aContext,aFontMetric);
	_spaceAfter.setFontMetric(aContext,aFontMetric);
	_leftMargin.setFontMetric(aContext,aFontMetric);
	_rightMargin.setFontMetric(aContext,aFontMetric);
    }

    protected final Dimension computeSize(ContextInterface aContext,
					  StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
					
					  StrategySetOrigin aSetOriginStrategy,
					  StrategyComputeSizeTill aComputeSizeTillStrategy) {
	if ( numberOfChildren() == 0 ) {
	    return new Size();
	}
	Point theFatherPoint = aContext.newPoint();
	Point thePoint = aContext.newPoint();
	Size theResult = new Size();
	GlyphInterface theCurrentSon = null;
	Dimension theCurrentSonSize = null;
	Dimension thePreviousSonSize = null;
	boolean theFirstFlag = true;
	PositionToFather thePositionToFather = aContext.newPositionToFather();
	PositionToBrother thePositionToBrother = null;
	final int theLeftMarginInPixel = _leftMargin.getValueInPixel();
	final int theRightMarginInPixel = _rightMargin.getValueInPixel();
	final int theSpaceBeforeInPixel = _spaceBefore.getValueInPixel();
	final int theSpaceAfterInPixel = _spaceAfter.getValueInPixel();
	int theInterWordInPixel = 0;
	for ( int theRank = 0; theRank < 3; theRank++ ) {
	    switch ( theRank ) {
	    case 0:
		theCurrentSonSize = new Dimension();
		theInterWordInPixel = theLeftMarginInPixel;
		break;
	    case 1:
		theCurrentSon = _child;
		theCurrentSonSize = aComputeSizeTillStrategy.getExactOrEstimatedSize(theCurrentSon,0);
		break;
	    case 2:
		theInterWordInPixel = theRightMarginInPixel;
		theCurrentSonSize = new Dimension(0,theSpaceBeforeInPixel,0,theSpaceAfterInPixel);
		break;
	    default:
		reportError(new ImplementationError("Unexpected"));
		break;
	    }
	    if ( theFirstFlag ) {
		thePositionToFather.movePointToPosition(thePoint,theFatherPoint);
		thePositionToBrother = aContext.newPositionToBrother(thePositionToFather);
		theFirstFlag = false;
	    }
	    else {
		thePositionToBrother.setDx(thePreviousSonSize.getExitPointX() + theInterWordInPixel);
		thePositionToBrother.setDy(thePreviousSonSize.getExitPointY());
		thePositionToBrother.movePointToPosition(thePoint);
	    }
	    if ( theRank == 1 ) {
		aSetOriginStrategy.doAlgorithm(theCurrentSon,thePoint.getX(),thePoint.getY());
		aComputeSizeTillStrategy.setFormatStatus(theCurrentSon,0);
	    }
	    theResult.growToIncludeSizeAt(theCurrentSonSize,thePoint);
	    thePreviousSonSize = theCurrentSonSize;
	}
	if ( thePreviousSonSize != null ) {
	    theResult.setExitPointX(thePoint.getX() + 
				    thePreviousSonSize.getExitPointX());
	    theResult.setExitPointY(thePoint.getY() + 
				    thePreviousSonSize.getExitPointY());
	}
	aContext.disposePoint(thePoint);
	aContext.disposePoint(theFatherPoint);
	aContext.disposePositionToFather(thePositionToFather);
	if ( thePositionToBrother != null ) {
	    aContext.disposePositionToBrother(thePositionToBrother);
	}
	return theResult;
    }

    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	return ( aDistance.getValue() - _leftMargin.getValueInPixel() );
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }

    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	return ( aGlyph.getSize().getExitPointX() + _rightMargin.getValueInPixel() + aDecalage.getValue() );
    }

    protected boolean sonModified(GlyphInterface aGlyph,
				  SonModificationStatus aStatus,
				  ContextInterface aContext) {
	final int theRank = indexOf(aGlyph);
	final boolean theNotCleanFlag = ( aStatus != SonModificationStatus.KILL );
	ModificationInterface theModification = getModification();
	if ( theModification.isSonModified() ) {
	    final int thePreviousRank = theModification.getSonModifiedRank();
	    if ( thePreviousRank == Modification.NO_RANK_SPECIFIED ) {
		theModification.setSonModifiedRank(theRank);
	    }
	    else {
		theModification.setSonModifiedRank(Math.min(theRank,thePreviousRank));
	    }
	}
	else {
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification(ModificationStatus.SON);
		setModification(theModification);
	    }
	    theModification.setSonModifiedRank(theRank);
	}
	if ( ! theNotCleanFlag ) {
	    theModification.setSonClean(true);
	}
	return true;
    }

    public final GlyphInterface getChild(int aRank) {
	if ( Globals.ROBUST ) {
	    if ( (aRank != 0) || (_child == null) ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	return _child;
    }
  
    public final void insertChild(int aRank, GlyphInterface aGlyph) {
	super.insertChild(aRank,aGlyph);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = aGlyph;
    }
  
    public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	super.replaceChild(aRank,aGlyph);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = aGlyph;
    }
  
    public final void removeChild(int aRank) {
	super.removeChild(aRank);
	if ( Globals.ROBUST ) {
	    if ( aRank != 0 ) {
		reportError(new GlyphOutOfBoundError(this,aRank));
	    }
	}
	_child = null;
    }
  
    public final int numberOfChildren() { 
	if ( _child != null ) {
	    return 1;
	}
	return 0;
    }
  
    public final String toString() {
	StringBuffer theResult = new StringBuffer("Insets<");
	theResult.append("before:").append(_spaceBefore).append(";")
	    .append("after:").append(_spaceAfter)
	    .append(";left:").append(_leftMargin).append(";right: ").append(_leftMargin);
	theResult.append(super.toString()).append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "Insets";
    }

    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" SpaceBefore=\"");
	anOutput.write(Integer.toString(_spaceBefore.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_spaceBefore.getUnit());
	anOutput.write(" SpaceAfter=\"");
	anOutput.write(Integer.toString(_spaceAfter.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_spaceAfter.getUnit());
	anOutput.write(" SpaceToLeft=\"");
	anOutput.write(Integer.toString(_leftMargin.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_leftMargin.getUnit());
	anOutput.write(" SpaceToRight=\"");
	anOutput.write(Integer.toString(_rightMargin.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_rightMargin.getUnit());
    }

    private GlyphInterface _child;
    private MeasureInterface _spaceBefore;
    private MeasureInterface _spaceAfter;
    private MeasureInterface _leftMargin;
    private MeasureInterface _rightMargin;
  
}
