//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.SelectionPathInterface;
import figue.path.LinearPath;
import figue.resource.FontMetricInterface;
import figue.geometry.Rectangle;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.ModificationInterface;
import figue.format.SonModificationStatus;
import figue.format.ModificationStatus;
import figue.format.Modification;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.format.ActionNewLine;
import figue.format.ActionShrinkSon;
import figue.format.ActionShrink;
import figue.error.ShouldNotBeCalledError;
import figue.MeasureInterface;
import figue.UnitInterface;
import figue.Measure;
import figue.internal.ContextInterface;
import figue.internal.VectorGlyph;
import figue.internal.PairGlyphPath;

import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] Paragraph.
 */

public class Paragraph extends DecoratedGlyphAbstract {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Paragraph.java,v $ $revision$ $Date: 1998/11/02 15:54:42 $ Copyright 1998 INRIA.";

    public Paragraph(MeasureInterface anInterWord,
		     MeasureInterface anIndentation,
		     MeasureInterface anInterline) {
	_interWord = anInterWord;
	_interline = anInterline;
	_indent = anIndentation;
	_children = new VectorGlyph(50,25);
	_endOfLines = new LineData();
	LineData theNewLine = new LineData(0,0,new Line(anInterWord,_children),_endOfLines,null);
	_component = new Column(anIndentation,anInterline,_children);
	theNewLine.getComponent().setParent(_component);
	_component.setParent(this);
	_component.setDatas(_endOfLines);
    }

    public Paragraph(int anInterWord, int anIndentation, int anInterline) {
	this(new Measure(anInterWord,UnitInterface.BLANK_TENTH),
	     new Measure(anIndentation,UnitInterface.BLANK_TENTH),
	     new Measure(anInterline,UnitInterface.BLANK_TENTH));
    }

    public Paragraph() {
	this(10,0,0);
    }

    public final Object clone() throws CloneNotSupportedException {
	final Paragraph theResult = (Paragraph) super.clone();
	theResult._interWord = (MeasureInterface)_interWord.clone();
	theResult._interline = (MeasureInterface)_interline.clone();
	theResult._indent = (MeasureInterface)_indent.clone();
	theResult._children = new VectorGlyph(numberOfChildren(),25);
	theResult._endOfLines = new LineData();
	LineData theNewLine = new LineData(0,0,new Line(theResult._interWord,theResult._children),theResult._endOfLines,null);
	theResult._component = new Column(theResult._indent,theResult._interline,theResult._children);
	theNewLine.getComponent().setParent(theResult._component);
	theResult._component.setParent(theResult);
	theResult._component.setDatas(theResult._endOfLines);
	final int theSize = numberOfChildren();
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theResult.addChild((GlyphInterface)getChild(theRank).clone());
	}
	return theResult;
    }
    
    public final MeasureInterface getInterWord() {
	return _interWord;
    }

    public final void setInterWord(MeasureInterface anInterWord) {
	_interWord = anInterWord;
    }

    public final void setInterWord(int anInterWord) {
	setInterWord(new Measure(anInterWord,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getInterline() {
	return _interline;
    }

    public final void setInterline(MeasureInterface anInterline) {
	_interline = anInterline;
    }

    public final void setInterline(int anInterline) {
	setInterline(new Measure(anInterline,UnitInterface.BLANK_TENTH));
    }

    public final MeasureInterface getIndentation() {
	return _indent;
    }

    public final void setIndentation(MeasureInterface anIndentation) {
	_indent = anIndentation;
    }

    public final void setIndentation(int anIndentation) {
	setIndentation(new Measure(anIndentation,UnitInterface.BLANK_TENTH));
    }

    protected final GlyphInterface getComponent() {
	return  _component;
    }

    protected final void setComponent(GlyphInterface aGlyph) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void executeActionNewLine(GlyphInterface aGlyph) 
	throws StopFormattingSonsException{
	insertBreakAfter(aGlyph);
	throw new StopFormattingSonsException(aGlyph.getParent());
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag) 
	throws StopFormattingSonsException {
	if ( aMustFormatFlag || isNew() ) {
	    flatAfter(0);
	}
	DistanceInterface theNewDistance = aDistance.lowerLevel(aContext,this);
	theNewDistance.checkBefore(anAlternative);
	getComponent().format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
	theNewDistance.newBrother(aContext,getComponent());
	theNewDistance.upperLevel(aContext);
    }

    public final boolean isActionNewLineComplient(ActionNewLine anAction) {
	return ( getChild(findLine(_children.indexOf(anAction.getPlace()) + 1).getBeginOfLine()) != anAction.getPlace() );
    }
  
    public final GlyphInterface getChild(int aRank) {
	return _children.elementAt(aRank);
    }
  
    public final void insertChild(int aRank, GlyphInterface aGlyph) {
	_children.insertElementAt(aGlyph,aRank);
	final LineData theLine = findLine(aRank);
	theLine.moveEndOfLine(+1);
	aGlyph.setParent(theLine.getComponent());
    }
  
    public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	_children.setElementAt(aGlyph,aRank);
	aGlyph.setParent(findLine(aRank).getComponent());
    }

    public final void removeChild(int aRank) {
	_children.removeElementAt(aRank);
	findLine(aRank).moveEndOfLine(-1);
    }

    private final void insertBreakAfter(GlyphInterface aGlyph) {
	final LineData theLine = findLine(_children.indexOf(aGlyph) + 1);
	final GlyphInterface theLineComponent = theLine.getComponent();
	final GlyphInterface theLines = theLineComponent.getParent();
	final int theLineRank = theLines.indexOf(theLineComponent);
	final int theLineRankUnder = theLineRank + 1;
	final int theColumnRank = _children.indexOf(aGlyph);
	LineData theLineUnder;
	GlyphInterface theLineUnderComponent;
	if ( theLineRankUnder == theLines.numberOfChildren() ) {
	    theLineUnder = new LineData(theColumnRank + 1,
					numberOfChildren(),
					new Line(_interWord,_children),
					theLine, null);
	    theLineUnderComponent = theLineUnder.getComponent();
	    theLines.insertChild(theLineRankUnder,theLineUnderComponent);
	    theLineUnderComponent.setParent(theLines);
	}
	else {
	    theLineUnder = findLine(theLine.getEndOfLine() + 1);
	    theLineUnderComponent = theLineUnder.getComponent();
	    theLineUnderComponent.setNew();
	}
	GlyphInterface theCurrentSon;
	for ( int theRank = theLine.getEndOfLine() - 1; theRank > theColumnRank; theRank-- ) {
	    theCurrentSon = getChild(theRank);
	    theCurrentSon.removeDependent();
	    theCurrentSon.setParent(theLineUnderComponent);
	}
	final int theCount = theLine.getEndOfLine() - 1 - theColumnRank;
	theLine.setEndOfLine(theLine.getEndOfLine() - theCount);
	theLineUnder.setBeginOfLine(theLine.getEndOfLine());
    }

    private final void flatAfter(int aRank) {
	LineData theLine = findLine(aRank + 1);
	if ( theLine != null ) {
	    LineData theLast = theLine;
	    theLine.getComponent().setNew();
	    final GlyphInterface theComponent = theLine.getComponent();
	    while ( theLast.getNextLine() != null ) {
		getComponent().removeChild(-1);
		theLast = theLast.getNextLine();
	    }
	    if ( theLast != theLine ) {
		theLine.setEndOfLine(theLast.getEndOfLine());
		theLine.setNextLine(null);
	    }
	    final int theSize = numberOfChildren();
	    GlyphInterface theCurrentSon;
	    for ( int theRank = aRank; theRank < theSize; theRank++ ) {
		theCurrentSon = getChild(theRank);
		theCurrentSon.setParent(theComponent);
		theCurrentSon.removeDependent();
	    }
	}
    }
  
    public final int numberOfChildren() { 
	return _children.size(); 
    }

    private final LineData findLine(int aRank) {
	LineData theCurrentLine = _endOfLines;
	while ( true ) {
	    if ( ( theCurrentLine.getBeginOfLine() <= aRank ) &&
		 ( theCurrentLine.getEndOfLine() > aRank ) || 
		 ( theCurrentLine.getNextLine() == null ) ) {
		return theCurrentLine;
	    }
	    theCurrentLine = theCurrentLine.getNextLine();
	}
    }

    protected final PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	return getParent().computePathTo(this,aPath);
    }

    public final PairGlyphPath getDeepestContains(int x,
						  int y, 
						  Rectangle aBoundingBox,
						  ContextInterface aContext) {
	PairGlyphPath theResult = super.getDeepestContains(x,y,aBoundingBox,aContext);
	if ( theResult != null ) {
	    final PathInterface thePath = theResult.getPath();
	    if ( thePath == null ) {
		theResult.setGlyph(this);
	    }
	    else {
		if ( thePath.numberOfChildren() == 0 ) {
		    theResult.setGlyph(this);
		    theResult.setPath(null);
		}
		else {
		    final PathInterface theLowerPath = thePath.getChild(0);
		    theLowerPath.setRank(_component.findLine(thePath.getRank()).getBeginOfLine() + theLowerPath.getRank());
		    theResult.setPath(theLowerPath);
		}
	    }
	}
	return theResult;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Paragraph<");
	LineData theCurrentLine = _endOfLines;
	while ( theCurrentLine.getNextLine() != null ) {
	    theCurrentLine = theCurrentLine.getNextLine();
	    final int theSize = theCurrentLine.getEndOfLine();
	    boolean theFirstFlag = true;
	    for ( int theRank = theCurrentLine.getBeginOfLine(); theRank < theSize; theRank++ ) {
		if ( theFirstFlag ) { 
		    theFirstFlag = false;
		} 
		else {
		    theResult.append(",");
		}
		theResult.append(theRank).append(":").append(getChild(theRank).toString());
	    }
	    theResult.append(" [EOL] ");
	}
	theResult.append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "P";
    }

    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" Interword=\"");
	anOutput.write(Integer.toString(_interWord.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interWord.getUnit());
	anOutput.write(" Interline=\"");
	anOutput.write(Integer.toString(_interline.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interline.getUnit());
	anOutput.write(" Indent=\"");
	anOutput.write(Integer.toString(_indent.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_indent.getUnit());
    }
    
    protected final boolean writeExtrinsicState(Writer anOutput) throws IOException {
	LineData theCurrentLine = _endOfLines;
	boolean theNonEmptyFlag = false;
	while ( theCurrentLine.getNextLine() != null ) {
	    theCurrentLine = theCurrentLine.getNextLine();
	    final int theSize = theCurrentLine.getEndOfLine();
	    for ( int theRank = theCurrentLine.getBeginOfLine(); theRank < theSize; theRank++ ) {
		if ( ! theNonEmptyFlag ) { 
		    anOutput.write(">");
		}
		theNonEmptyFlag = true;
		anOutput.write(' ');
		getChild(theRank).writeState(anOutput);
	    }
	}
	return theNonEmptyFlag;
    }
	
    private MeasureInterface _interWord;
    private MeasureInterface _interline;
    private MeasureInterface _indent;
    private VectorGlyph _children;
    private LineData _endOfLines;
    private Column _component;
  
}

final class Column extends VerticalAbstract {

    Column(MeasureInterface anIndentation,
	   MeasureInterface anInterline,
	   VectorGlyph aChildrenList) {
	super(anIndentation,anInterline);
	_head = null;
	_children = aChildrenList;
	_size = 0;
    }

    public final Object clone() throws CloneNotSupportedException {
	reportError(new ShouldNotBeCalledError());
	return null;
}

    public final SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
							int aRank) {
	return aSelectionPath;
    }
    
    public final void doTranslatesZones(Rectangle aRefreshZone,
					Rectangle aVisibleZone,
					int aRank) {
    super.doTranslatesZones(aRefreshZone,aVisibleZone,aRank);
    findLineForRank(aRank).getComponent().doTranslatesZones(aRefreshZone,aVisibleZone,-1);
    }
    public final void undoTranslateZones(Rectangle aRefreshZone,
					 Rectangle aVisibleZone,
					 int aRank) {
	super.undoTranslateZones(aRefreshZone,aVisibleZone,aRank);
	findLineForRank(aRank).getComponent().undoTranslateZones(aRefreshZone,aVisibleZone,-1);
    }
    
    public final void tryAlternativeForDistance(DistanceInterface aDistance,
						AlternativeInterface anAlternative) {
	aDistance.getDistance().tryAlternative(anAlternative);
    }
    
    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	;
    }
    
    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	final ModificationInterface theModification = getModification();
	if ( theModification.isSonModified() ) {
	    final int theRank = theModification.getSonModifiedRank();
	    if ( theRank >= 0 ) {
		flatAfter(theRank,aContext);
	}
	}
	super.format2D(aContext,aFontMetric,anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
    }

private final void flatAfter(int aRank, ContextInterface aContext) {
    LineData theLine  = findLine(aRank);
    if ( theLine != null ) {
	LineData theLast = theLine;
	final int theLastEndRank = theLast.getEndOfLine() - 1;
	if ( theLastEndRank >= 0 ) {
	    final GlyphInterface theComponent = theLine.getComponent();
	    ModificationInterface theModification = theComponent.getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification(ModificationStatus.SON);
		theModification.setSonModifiedRank(theLastEndRank - theLast.getBeginOfLine());
		theComponent.setModification(theModification);
	    }
	    while ( theLast.getNextLine() != null ) {
		removeChild(-1);
		theLast = theLast.getNextLine();
	    }
	    if ( theLast != theLine ) {
		theLine.setEndOfLine(theLast.getEndOfLine());
		theLine.setNextLine(null);
	    }
	    final GlyphInterface theParent = getParent();
	    final int theSize = theParent.numberOfChildren();
	    GlyphInterface theCurrentSon;
	    for ( int theRank = theLastEndRank; theRank < theSize; theRank++ ) {
		theCurrentSon = theParent.getChild(theRank);
		theCurrentSon.setParent(theComponent);
		theCurrentSon.removeDependent();
	    }
	}
    }
}

    protected final boolean sonModified(GlyphInterface aGlyph,
					SonModificationStatus aStatus,
					ContextInterface aContext) {
	if ( aStatus != SonModificationStatus.EXPAND ) {
	    ModificationInterface theModification = getModification();
	    if ( theModification.isSonModified() ) {
		int theIndex = indexOf(aGlyph);
		if ( theIndex > 0 ) {
		    final ModificationInterface theSonModification = aGlyph.getModification();
		    if ( theSonModification.isSonModified() &&
			 ( theSonModification.getSonModifiedRank() == 0 ) ) {
			theIndex+= -1;
		    }
		}
		final int thePreviousRank = theModification.getSonModifiedRank();
		if ( thePreviousRank == Modification.NO_RANK_SPECIFIED ) {
		    theModification.setSonModifiedRank(theIndex);
		}
		else {
		    theModification.setSonModifiedRank(Math.min(theIndex,thePreviousRank));
		}
	    }
	    else {
		if ( ! theModification.isModified() ) {
		    theModification = aContext.newModification(ModificationStatus.SON);
		    setModification(theModification);
		}
		int theIndex = indexOf(aGlyph);
		if ( theIndex > 0 ) {
		    final ModificationInterface theSonModification = aGlyph.getModification();
		    if ( theSonModification.isSonModified() &&
			 ( theSonModification.getSonModifiedRank() == 0 ) ) {
			theIndex+= -1;
		    }
		}
		theModification.setSonModifiedRank(theIndex);
	    }
	}
	return true;
    }

    public final void setDatas(LineData aDataSet) {
	_head = aDataSet;
	if ( _head.getNextLine() != null ) {
	    _size = 1;
	}
	else {
	    _size = 0;
	}
    }

public final GlyphInterface getChild(int aRank) {
    return findLine(aRank).getComponent();
}
  
public final void insertChild(int aRank, GlyphInterface aGlyph) {
    _size++;
}
  
public final void replaceChild(int aRank, GlyphInterface aGlyph) {
    ;
}

public final void removeChild(int aRank) {
    _size--;
}
  
public final int numberOfChildren() { 
    return _size;
}

public  final LineData findLine(int aRank) {
    LineData theCurrentLine = _head;
    int theCounter = -1;
    while ( ( aRank > theCounter ) && ( theCurrentLine != null ) ) {
	theCounter++;
	theCurrentLine = theCurrentLine.getNextLine();
    }
    return theCurrentLine;
}

private final LineData findLineForRank(int aRank) {
    LineData theCurrentLine = _head;
    while ( theCurrentLine.getNextLine() != null ) {
	theCurrentLine = theCurrentLine.getNextLine();
	if ( ( theCurrentLine.getBeginOfLine() <= aRank ) &&
	     ( theCurrentLine.getEndOfLine() > aRank ) ) {
	    return theCurrentLine;
	}
    }
    return null;
}

protected final PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
    return getParent().computePathTo(this,aPath);
}
    
private LineData _head;
private VectorGlyph _children;
private int _size;
  
}

final class Line extends HorizontalAbstract {

    Line(MeasureInterface anInterword,
	 VectorGlyph aChildrenList) {
	super(anInterword);
	_datas = null;
	_children = aChildrenList;
    }

    public final Object clone() throws CloneNotSupportedException {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    protected SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
						     int aRank) {
	final int theRank = _datas.getBeginOfLine() + aRank;
	final int theSize = aSelectionPath.numberOfChildren();
	SelectionPathInterface theCurrentSon;
	final int theEnd = _datas.getEndOfLine();
	for ( int theCounter = 0; theCounter < theSize; theCounter++ ) {
	    theCurrentSon = aSelectionPath.getChild(theCounter);
	    if ( theCurrentSon.getRank() == theRank ) {
		return theCurrentSon;
	    }
	    else {
		if ( theCurrentSon.getRank() >= theEnd ) {
		    break;
		}
	    }
	}
	return null;
    }
  
    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	anAlternative.tryBetter(new ActionNewLine(aDecalage.getBrother()),
				getParent().getParent(),
				aDecalage);
	anAlternative.tryBetter(ActionShrink.INSTANCE,this,aDecalage);
	DecalageInterface theDecalage = aDecalage;
	while ( theDecalage.hasBrother() ) {
	    anAlternative.tryBetter(ActionShrinkSon.INSTANCE,
				    theDecalage.getBrother(),
				    theDecalage);
	    theDecalage = theDecalage.getDecalage();
	}
    }

    public void setDatas(LineData aDataSet) {
	_datas = aDataSet;
    }

    public final GlyphInterface getChild(int aRank) {
	return _children.elementAt(aRank + _datas.getBeginOfLine());
    }
  
    public final void insertChild(int aRank, GlyphInterface aGlyph) {
	super.insertChild(aRank,aGlyph);
	_children.insertElementAt(aGlyph,aRank + _datas.getBeginOfLine());
	_datas.moveEndOfLine(+1);
    }
  
    public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	super.replaceChild(aRank,aGlyph);
	_children.setElementAt(aGlyph,aRank + _datas.getBeginOfLine());
    }

    public final void removeChild(int aRank) {
	super.removeChild(aRank);
	_children.removeElementAt(aRank + _datas.getBeginOfLine());
	_datas.moveEndOfLine(-1);
    }
  
    public final int numberOfChildren() { 
	return ( _datas.getEndOfLine() - _datas.getBeginOfLine() ); 
    }

    protected final PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	final PathInterface theResult = new LinearPath(indexOf(aGlyph) + _datas.getBeginOfLine());
	if ( aPath != null ) {
	    theResult.addChild(aPath);
	}
	return getParent().computePathTo(this,theResult);
    }

    private LineData _datas;
    private VectorGlyph _children;
    
}

final class LineData {
    
    public LineData(int aFirstRank, int aLastRank, Line aLine, LineData aPreviousLine, LineData aNextLine) {
	_beginOfLine = aFirstRank;
	_endOfLine = aLastRank;
	_previousLine = aPreviousLine;
	_nextLine = aNextLine;
	_component = aLine;
	if ( _component != null ) {
	    _component.setDatas(this);
	}
	if ( _previousLine != null ) {
	    _previousLine._nextLine = this;
	}
	if ( _nextLine != null ) {
	    _nextLine._previousLine = this;
	}
    }

    public LineData() {
	this(-1,-1,null,null,null);
    }

    public final int getBeginOfLine() {
	return _beginOfLine;
    }

    public final void setBeginOfLine(int aValue) {
	_beginOfLine = aValue;
    }

    public final int getEndOfLine() {
	return _endOfLine;
    }

    public final void setEndOfLine(int aValue) {
	_endOfLine = aValue;
    }

    public final LineData getPreviousLine() {
	return _previousLine;
    }

    public final LineData getNextLine() {
	return _nextLine;
    }

    public final void setNextLine(LineData aLine) {
	_nextLine = aLine;
    }

    public final Line getComponent() {
	return _component;
    }

    public final void setComponent(Line aLine) {
	_component = aLine;
    }

    public final void moveEndOfLine(int aValue) {
	_endOfLine += aValue;
	LineData theCurrentLine = this;
	while ( theCurrentLine.getNextLine() != null ) {
	    theCurrentLine = theCurrentLine.getNextLine();
	    theCurrentLine._beginOfLine+=aValue;
	    theCurrentLine._endOfLine+=aValue;
	}
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("LineData<");
	theResult.append(_beginOfLine).append("/").append(_endOfLine);
	if ( _component != null ) {
	    theResult.append(", ").append(_component.toString());
	}
	theResult.append(">");
	return theResult.toString();
    }

    private int _beginOfLine;
    private int _endOfLine;
    private LineData _previousLine;
    private LineData _nextLine;
    private Line _component;
  
}
