//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import com.sun.javadoc.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.io.IOException;
import java.io.Writer;

public class IndexWriter extends HtmlWriter {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/IndexWriter.java,v $ $revision$ $Date: 1998/11/02 15:54:48 $ Copyright 1998 INRIA.";

    public IndexWriter(String anOutputDirectory,
		       String aTemplateDirectory,
		       Hashtable aKnownTypeList,
		       Hashtable anEntryList,
		       Vector anIndex) {
	super(anOutputDirectory,aTemplateDirectory,"allNames.html",aKnownTypeList);
	_index = anIndex;
	_entryList = anEntryList;
    }

    protected void document () throws IOException {
	final Writer theStream = getOutputStream();
	copyTemplateHeader(theStream);
	title("Index",theStream);
	final int theSize = _index.size();
	final Doc[] theIndex = new Doc[theSize];
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theIndex[theRank] = (Doc)_index.elementAt(theRank);
	}
	if ( theSize != 0 ) {
	    Utility.quickSort(theIndex,0,theSize - 1);
	}
	final String[] theEntryList = getEntryList(_entryList);
	final int theEntrySize = theEntryList.length;
	for ( int theRank = 0; theRank < theEntrySize; theRank++ ) {
	    letterEntry(theStream,theEntryList,theRank);
	    theStream.write("<UL>\n");
	    final String theCurrentLetter = theEntryList[theRank];
	    for ( int theIndexRank = 0; theIndexRank < theSize; theIndexRank++ ) {
		if ( theIndex[theIndexRank].name().substring(0,1).toUpperCase().equals(theCurrentLetter) ) {
		    theStream.write("<LI>");
		    fastLinkTo(theStream,theIndex[theIndexRank]);
		    theStream.write("\n");
		}
	    }
	    theStream.write("</UL>\n");
	    theStream.write("</TD></TR>\n");
	}
	copyTemplateFooter(theStream);
    }

    private String[] getEntryList(Hashtable anEntryList) {
	final String[] theResult = new String[anEntryList.size()];
	final Enumeration theKeys = anEntryList.keys();
	int theCounter = 0;
	while ( theKeys.hasMoreElements() ) {
	    theResult[theCounter++] = (String)(theKeys.nextElement());
	}
	Utility.quickSort(theResult,0,theResult.length-1);
	return theResult;
    }

    private void letterEntry(Writer aStream,String[] anEntryList,int aRank) 
	throws IOException {
	final int theSize = anEntryList.length;
	aStream.write("<TR><TD></TD>\n");
	aStream.write("<TD>\n<A NAME=\"" + anEntryList[aRank].toUpperCase() + "\"></A>\n");
	aStream.write("<H3><IMG SRC=\"../../images/box.gif\" ALT=\"*\"> ");
	for ( int theRank = 0; theRank < aRank; theRank++ ) {
	    aStream.write("<A HREF=\"#" + anEntryList[theRank] +
			  "\">" + anEntryList[theRank] + "</A> ");
	}
	aStream.write("<STRONG><FONT SIZE=\"+4\" COLOR=\"#ff0000\">" + anEntryList[aRank] + "</FONT></STRONG> ");
	for ( int theRank = aRank + 1; theRank < theSize; theRank++ ) {
	    aStream.write("<A HREF=\"#" + anEntryList[theRank] +
			  "\">" + anEntryList[theRank] + "</A> ");
	}
	aStream.write("</H3>\n");
    }

    private void fastLinkTo(Writer aStream, Doc anObject) throws IOException {
	try {
	    if ( anObject.isInterface() ) {
		fastLinkToClass(aStream,(ClassDoc)anObject,true);
	    }
	    else {
		if ( anObject.isException() ) {
		    fastLinkToClass(aStream,(ClassDoc)anObject,true);
		}
		else {
		    if ( anObject.isError() ) {
			fastLinkToClass(aStream,(ClassDoc)anObject,true);
		    }
		    else {
			if ( anObject.isClass() ) {
			    fastLinkToClass(aStream,(ClassDoc)anObject,true);
			}
			else {
			    if ( anObject.isConstructor() ) {
				fastLinkToConstructor(aStream,(ConstructorDoc)anObject,true);
			    }
			    else {
				if ( anObject.isField() ) {
				    fastLinkToVariable(aStream,(FieldDoc)anObject,true);
				}
				else {
				    if ( anObject.isMethod() ) {
					fastLinkToMethod(aStream,(MethodDoc)anObject,true);
				    }
				    else {
					fastLinkToPackage(aStream,(PackageDoc)anObject,true);
				    }
				}
			    }
			}
		    }
		}
	    }
	}
	catch ( ClassCastException anException ) {
	    anException.printStackTrace();
	    System.exit(-1);
	}
    }

    private void fastLinkToPackage(Writer aStream, PackageDoc aPackage, boolean aFlag)
	throws IOException {
	aStream.write("<A HREF=\"Package-" + aPackage.name() + ".html\">" +
		      aPackage.name() + "</A>");
	if ( aFlag ) {
	    aStream.write("- Package");
	}
    }

    private void fastLinkToClass(Writer aStream, ClassDoc aClass, boolean aFlag)
	throws IOException {
	aStream.write("<A HREF=\"" + aClass.qualifiedName() + ".html\">" +
		      aClass.name() + "</A>");
	if ( aFlag ) {
	    aStream.write(" - Class in package ");
	    fastLinkToPackage(aStream,aClass.containingPackage(),false);
	}
    }

    private void fastLinkToConstructor(Writer aStream, ConstructorDoc aConstructor, boolean aFlag)
	throws IOException {
	aStream.write("<A HREF=\"" + aConstructor.containingClass().qualifiedName()
		      + ".html#" + aConstructor.name() + aConstructor.signature() + "\">" +
		      aConstructor.name() + "</A>" + aConstructor.flatSignature());
	if ( aFlag ) {
	    aStream.write(" - Constructor in class ");
	    fastLinkToClass(aStream,aConstructor.containingClass(),false);
	}
    }

    private void fastLinkToMethod(Writer aStream, MethodDoc aMethod, boolean aFlag)
	throws IOException {
	aStream.write("<A HREF=\"" + aMethod.containingClass().qualifiedName()
		      + ".html#" + aMethod.name() + aMethod.signature() + "\">" +
		      aMethod.name() + "</A>" + aMethod.flatSignature());
	if ( aFlag ) {
	    aStream.write(" - Method in ");
	    final ClassDoc theOwner = aMethod.containingClass();
	    if ( theOwner.isInterface() ) {
		aStream.write("interface ");
	    }
	    else {
		aStream.write("class ");
	    }
	    fastLinkToClass(aStream,theOwner,false);
	}
    }

    private void fastLinkToVariable(Writer aStream, FieldDoc aVariable, boolean aFlag)
	throws IOException {
	aStream.write("<A HREF=\"" + aVariable.containingClass().qualifiedName()
		      + ".html#" + aVariable.name() + "\">" +
		      aVariable.name() + "</A>");
	if ( aFlag ) {
	    aStream.write("- Variable in ");
	    final ClassDoc theOwner = aVariable.containingClass();
	    if ( theOwner.isInterface() ) {
		aStream.write("interface ");
	    }
	    else {
		aStream.write("class ");
	    }
	    fastLinkToClass(aStream,theOwner,false);
	}
    }
    
    public String getOutputFileName() {
	return "allNames.html";
    }

    private Vector _index;
    private Hashtable _entryList;
    
}
