//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.event;

import figue.CommandInterface;
import figue.InteractiveFacade;
import figue.Priority;
import figue.path.ExtendSelection;
import figue.path.InverseSelection;
import figue.path.Path;
import figue.path.ShrinkSelection;
import figue.path.MarkGlyph;
import figue.path.PathInterface;
import figue.path.PathOperationInterface;
import figue.path.SelectionPathInterface;
import figue.resource.BasicColor;
import figue.resource.WrongResourceException;
import figue.selection.SelectionInterface;
import figue.selection.WrongSelectionException;
import figue.error.ImplementationError;
import figue.internal.InteractiveClientInterface;

/**
 * [PENDING: update the comments] DefaultListener.
 */

public abstract class DefaultListener {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/event/DefaultListener.java,v $ $revision$ $Date: 1998/11/02 15:54:53 $ Copyright 1998 INRIA.";

    protected DefaultListener(InteractiveFacade aFacade) {
	_facade = aFacade;
	_client = _facade.getClient();
	_currentSelection = null;
	final class CommandHome extends InteractiveCommand {
	    protected CommandHome(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoHome();
		return true;
	    }
	}
	_commandHome = new CommandHome(getClient());
	final class CommandEnd extends InteractiveCommand {
	    protected CommandEnd(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoEnd();
		return true;
	    }
	}
	_commandEnd = new CommandEnd(getClient());
	final class CommandUp extends InteractiveCommand {
	    protected CommandUp(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoUp();
		return true;
	    }
	}
	_commandUp = new CommandUp(getClient());
	final class CommandDown extends InteractiveCommand {
	    protected CommandDown(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoDown();
		return true;
	    }
	}
	_commandDown = new CommandDown(getClient());
	final class CommandLeft extends InteractiveCommand {
	    protected CommandLeft(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoLeft();
		return true;
	    }
	}
	_commandLeft = new CommandLeft(getClient());
	final class CommandRight extends InteractiveCommand {
	    protected CommandRight(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoRight();
		return true;
	    }
	}
	_commandRight = new CommandRight(getClient());
	final class CommandPageUp extends InteractiveCommand {
	    protected CommandPageUp(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoPageUp();
		return true;
	    }
	}
	_commandPageUp = new CommandPageUp(getClient());
	final class CommandPageDown extends InteractiveCommand {
	    protected CommandPageDown(InteractiveClientInterface aClient) {
		super(aClient);
	    }
	    public final boolean execute() {
		getReceiver().actionGoPageDown();
		return true;
	    }
	}
	_commandPageDown = new CommandPageDown(getClient());
	_commandExit = new CommandInterface() {
	    public boolean execute() {
		_facade.disposeWhenInactive();
		return true;
	    }
	};
    }
  
    protected final InteractiveFacade getFacade() {
	return _facade;
    }
  
    protected final SelectionInterface getCurrentSelection() {
	if ( _currentSelection == null ) {
	    try {
		_currentSelection = _facade.getSelection("CURRENT");
	    }
	    catch ( WrongSelectionException anException ) {
		try {
		    _currentSelection 
			= _facade.doGetSelection("CURRENT",
						 Priority.HIGH,
						 null,
						 _facade.doGetColor("Current",
								    BasicColor.GREEN));
		}
		catch ( WrongSelectionException anotherException ) {
		    throw new ImplementationError("Unexpected");
		}
		catch ( WrongResourceException anotherException ) {
		    throw new ImplementationError("Unexpected");
		}
	    }
	}
	return _currentSelection;
    }
  
    protected final void applyToGlyphAndRedisplay(PathInterface aPath) {
	_facade.updateAndRedisplay(aPath);
    }

    protected final void applyToSelectionAndRedisplay(PathInterface aPath) {
	_facade.updateSelectionAndRedisplay(aPath);
    }

    protected final void setCurrentSelection(PathInterface aPath) {
	addOperationOnLeaves(aPath,new ExtendSelection(getCurrentSelection()));
	PathInterface theUnselectPath = computeUnselectPath(_facade.getSelectionPath(),
							    getCurrentSelection());
	if ( theUnselectPath != null ) {
	    applyToSelectionAndRedisplay(merge(theUnselectPath,aPath));
	}
	else {
	    applyToSelectionAndRedisplay(aPath);
	}
    }

    protected final void modifyCurrentSelection(PathInterface aPath) {
	addOperationOnLeaves(aPath,new InverseSelection(getCurrentSelection()));
	applyToSelectionAndRedisplay(aPath);
    }

    public final void addOperationOnLeaves(PathInterface aPath, /// TEMPO
					   PathOperationInterface anOperation) {
	final int theSize = aPath.numberOfChildren();
	if ( theSize == 0 ) {
	    aPath.addOperation((PathOperationInterface)(anOperation.clone()));
	}
	else {
	    for( int theRank = 0; theRank < theSize; theRank++ ) {
		addOperationOnLeaves(aPath.getChild(theRank),anOperation);
	    }
	}
    }

    public final PathInterface merge(PathInterface aPath, /// TEMPO
				     PathInterface anotherPath) {
	final int theSize = anotherPath.numberOfOperation();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    aPath.addOperation(anotherPath.getOperation(theRank));
	}
	final int theChildrenSize = anotherPath.numberOfChildren();
	PathInterface theCurrentSon;
	int theCurrentSonRank;
	int theMergeRank = 0;
	PathInterface theMergeSon = null;
	for( int theRank = 0; theRank < theChildrenSize; theRank++ ) {
	    theCurrentSon = anotherPath.getChild(theRank);
	    theCurrentSonRank = theCurrentSon.getRank();
	    final int theCurrentSize = aPath.numberOfChildren();
	    while ( theMergeRank < theCurrentSize ) {
		theMergeSon = aPath.getChild(theMergeRank);
		if ( theMergeSon.getRank() == theCurrentSonRank ) {
		    break;
		}
		else {
		    theMergeSon = null;
		    theMergeRank++;
		}
	    }
	    if ( theMergeSon == null ) {
		theMergeSon = new Path(theCurrentSonRank);
		aPath.insertChild(theMergeSon,theMergeRank);
	    }
	    merge(theMergeSon,theCurrentSon);
	}
	return aPath;
    }
  
    public final PathInterface computeUnselectPath(SelectionPathInterface aSelectionPath, /// TEMPO
						    SelectionInterface aSelection) {
	PathInterface thePath = new Path(aSelectionPath.getRank());
	boolean theFindFlag = false;
	if ( aSelectionPath.contains(aSelection) ) {
	    theFindFlag = true;
	    thePath.addOperation(new ShrinkSelection(aSelection));
	}
	final int theSize = aSelectionPath.numberOfChildren();
	PathInterface theCurrentResult = null;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentResult = computeUnselectPath(aSelectionPath.getChild(theRank),aSelection);
	    if ( theCurrentResult != null ) {
		theFindFlag = true;
		thePath.addChild(theCurrentResult);
	    }
	}
	if ( theFindFlag ) {
	    return thePath;
	}
	else {
	    return null;
	}    
    }

    protected final CommandInterface doGetCommandHome() {
	return _commandHome;
    }
  
    protected final CommandInterface doGetCommandEnd() {
	return _commandEnd;
    }
  
    protected final CommandInterface doGetCommandUp() {
	return _commandUp;
    }
  
    protected final CommandInterface doGetCommandDown() {
	return _commandDown;
    }
  
    protected final CommandInterface doGetCommandLeft() {
	return _commandLeft;
    }
  
    protected final CommandInterface doGetCommandRight() {
	return _commandRight;
    }
  
    protected final CommandInterface doGetCommandPageUp() {
	return _commandPageUp;
    }
  
    protected final CommandInterface doGetCommandPageDown() {
	return _commandPageDown;
    }


    protected final CommandInterface doGetCommandExit() {
	return _commandExit;
    }

    protected final CommandInterface doGetCommandScrollToSelection() {
	return new CommandInterface() {
	    public final boolean execute() {
		insureVisible();
		return true;
	    }
	};
    }

    final void insureVisible() {
	_facade.insureVisible(getPathForSelection(getCurrentSelection(),_facade.getSelectionPath()));
    }
  
    private final static PathInterface getPathForSelection(SelectionInterface aSelection,
							   SelectionPathInterface aSelectionPath) {
	PathInterface theResult = null;
	if ( aSelectionPath.contains(aSelection) ) {
	    theResult = new Path();
	    theResult.addOperation(MarkGlyph.INSTANCE);
	}
	final int theSize = aSelectionPath.numberOfChildren();
	SelectionPathInterface theCurrentSon;
	PathInterface theResultForSon;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSon = aSelectionPath.getChild(theRank);
	    theResultForSon = getPathForSelection(aSelection,theCurrentSon);
	    if ( theResultForSon != null ) {
		theResultForSon.setRank(theCurrentSon.getRank());
		if ( theResult == null ) {
		    theResult = new Path();
		}
		theResult.addChild(theResultForSon);
	    }
	}
	return theResult;
    }

    protected final InteractiveClientInterface getClient() {
	return _client;
    } 

    private InteractiveFacade _facade;
    private InteractiveClientInterface _client;
    private SelectionInterface _currentSelection;
    private CommandInterface _commandHome;
    private CommandInterface _commandEnd;
    private CommandInterface _commandUp;
    private CommandInterface _commandDown;
    private CommandInterface _commandLeft;
    private CommandInterface _commandRight;
    private CommandInterface _commandPageUp;
    private CommandInterface _commandPageDown;
    private CommandInterface _commandExit;
  
    abstract class InteractiveCommand implements CommandInterface {
    
	protected InteractiveCommand(InteractiveClientInterface aClient) {
	    _receiver = aClient;
	}
    
	protected final InteractiveClientInterface getReceiver() {
	    return _receiver;
	}
    
	private final InteractiveClientInterface _receiver;
    
    }
  
}

