//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.internal.Globals;
import figue.error.ShouldNotBeCalledError;

public final class Alternative implements AlternativeInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/Alternative.java,v $ $revision$ $Date: 1998/11/02 15:54:59 $ Copyright 1998 INRIA.";
    
    public Alternative() {
	_glyph = null;
	_decalage = null;
	_action = ActionNone.INSTANCE;
    }
    
    public final void execute() 
	throws TooSmallPageException, StopFormattingSonsException {
	if ( Globals.DEBUG_FORMAT_HISTORY ) {
	    System.out.println("-> execute : " + toString()); 
	}
	_action.execute(_glyph,_decalage,this);
    }
    
    public final void tryBetter(ActionInterface anAction,
				GlyphInterface aGlyph,
				DecalageInterface aDecalage) {
	if ( anAction.isComplient(aGlyph) && 
	     anAction.isBetterThan(_action) ) {
	    _glyph = aGlyph;
	    _decalage = aDecalage;
	    _action = anAction;
	}
    }
    
    public final void setAction(ActionInterface anAction) {
	_action = anAction;
    }
    
    public final void preComputeActionCostNewLine(ActionNewLine anAction) {
	throw new ShouldNotBeCalledError();
    }
    
    public final void dispose() {
	_glyph = null;
	_decalage = null;
	_action = null;
    }
    
    public final String toString() {
	StringBuffer theResult = new StringBuffer("Alternative<");
	if ( _action != null ) {
	    theResult.append(_action);
	}
	if ( _glyph != null ) {
	    theResult.append(_glyph.toString());
	}
	if ( _decalage != null ) {
	    theResult.append(_decalage);
	}
	theResult.append(">");
	return theResult.toString();
    }
    
    private GlyphInterface _glyph;
    private DecalageInterface _decalage;
    private ActionInterface _action;
    
}
