//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.internal.ContextInterface;
import figue.internal.Globals;
import figue.error.ShouldNotBeCalledError;

public class LazyDecalage extends Value implements DecalageInterface, ComputedValueInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/LazyDecalage.java,v $ $revision$ $Date: 1998/11/02 15:55:03 $ Copyright 1998 INRIA.";
  
    public LazyDecalage(GlyphInterface aFather,
			DecalageInterface aDecalage) {
	super();
	_lastValue = -1;
	_father = aFather;
	_decalage = aDecalage;
    }
  
    public LazyDecalage(GlyphInterface aFather) {
	this(aFather,null);
    }
  
    public void init(GlyphInterface aFather,DecalageInterface aDecalage) {
	super.init();
	_lastValue = -1;
	_father = aFather;
	_decalage = aDecalage;
    }
  
    public final int getValue() {
	if ( _lastValue == -1 ) {
	    update();
	}
	return _lastValue;
    }

    public final int getEagerValue() {
	updateEager();
	return _lastValue;
    }
  
    public final void update() {
	_lastValue = evalValue();
    }

    public final void updateEager() {
	_lastValue = evalValue();
	notifyEagerChange();
    }

    protected int evalValue() {
	return _father.computeInitialDecalage2D(this);
    }

    public final GlyphInterface getFather() {
	return _father;
    }
  
    public final DecalageInterface getDecalage() {
	return _decalage;
    }
  
    public final void setDecalage(DecalageInterface aDecalage) {
	_decalage = aDecalage;
    }

    public void setBrother(GlyphInterface aGlyph) {
	//// BEURK !!
	throw new ShouldNotBeCalledError();
    }

    public boolean hasBrother() {
	return false;
    }

    public GlyphInterface getBrother() {
	//// BEURK !!
	throw new ShouldNotBeCalledError();
    }
  
    public void tryAlternative(AlternativeInterface anAlternative) {
	if ( Globals.DEBUG_FORMAT_HISTORY ) {
	    System.out.println("LazyDecalage.tryAlternative"); 
	}
	getFather().tryAlternativeForInitial(anAlternative);
    }

    public void dispose(ContextInterface aContext) {
	reinit();
	aContext.disposeLazyDecalage(this);
    }

    final void reinit() {
	_lastValue = -1;
	_father = null;
	_decalage = null;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("LazyDecalage<");
	theResult.append(_lastValue);
	if ( hasBrother() ) {
	    theResult.append(", brother " + getBrother().toString());
	}
	if ( _decalage != null ) {
	    theResult.append(_decalage);
	}
	theResult.append(">");
	return theResult.toString();
    }
  
    private int _lastValue;
    private GlyphInterface _father;
    private DecalageInterface _decalage;
    
}
