//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

public final class ModificationStatus {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/ModificationStatus.java,v $ $revision$ $Date: 1998/11/02 15:55:05 $ Copyright 1998 INRIA.";
  
    private ModificationStatus(String aName,
			       boolean anOriginFlag,
			       boolean aSizeFlag,
			       boolean aSonFlag) {
	_name = aName;
	_originModified = anOriginFlag;
	_sizeModified = aSizeFlag;
	_sonModified = aSonFlag;
	_nextStateWhenOriginModified = null;
	_nextStateWhenSizeModified = null;
    }
  
    public final boolean isModified() {
	return ( isOriginModified() || isSizeModified() || isSonModified() );
    }
  
    public final boolean isOriginModified() {
	return _originModified;
    }
  
    public final boolean isSizeModified() {
	return _sizeModified;
    }

    public final boolean isSonModified() {
	return _sonModified;
    }
  
    public final ModificationStatus originModified() {
	return _nextStateWhenOriginModified;
    }
  
    public final ModificationStatus sizeModified() {
	return _nextStateWhenSizeModified;
    }

    public final String toString() {
	return _name;
    }
  
    private String _name;
    private boolean _originModified;
    private boolean _sizeModified;
    private boolean _sonModified;
    private ModificationStatus _nextStateWhenOriginModified;
    private ModificationStatus _nextStateWhenSizeModified;

    public final static ModificationStatus NONE = new ModificationStatus("none",false,false,false);
    public final static ModificationStatus ORIGIN = new ModificationStatus("origin",true,false,false);
    public final static ModificationStatus SIZE = new ModificationStatus("size",false,true,false);
    public final static ModificationStatus ORIGIN_AND_SIZE = new ModificationStatus("origin&size",true,true,false);
    public final static ModificationStatus SON = new ModificationStatus("son",false,false,true);

    static {
	NONE._nextStateWhenOriginModified = ORIGIN;
	NONE._nextStateWhenSizeModified = SIZE;
	ORIGIN._nextStateWhenOriginModified = ORIGIN;
	ORIGIN._nextStateWhenSizeModified = ORIGIN_AND_SIZE;
	SIZE._nextStateWhenOriginModified = ORIGIN_AND_SIZE;
	SIZE._nextStateWhenSizeModified = SIZE;
	ORIGIN_AND_SIZE._nextStateWhenOriginModified = ORIGIN_AND_SIZE;
	ORIGIN_AND_SIZE._nextStateWhenSizeModified = ORIGIN_AND_SIZE;
	SON._nextStateWhenOriginModified = ORIGIN;
	SON._nextStateWhenSizeModified = SIZE;
    }
  
}
