//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.geometry.Dimension;
import figue.internal.ContextInterface;

public abstract class StrategySetSize {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategySetSize.java,v $ $revision$ $Date: 1998/11/02 15:55:08 $ Copyright 1998 INRIA.";
  
    protected StrategySetSize(ContextInterface aContext) {
	_context = aContext;
    }
    protected StrategySetSize() {
	this(null);
    }
  
    public abstract void doAlgorithm(GlyphInterface aGlyph,
				     Dimension aSize);

    public abstract StrategySetSize clone(ContextInterface aContext);

    protected final ContextInterface getContext() {
	return _context;
    }

    public final void setContext(ContextInterface aContext) {
	_context = aContext;
    }

    private ContextInterface _context;
  
    public final static StrategySetSize INCREMENTAL = new StrategySetSizeIncremental();
  
    public final static StrategySetSize NO_INCREMENTAL = new StrategySetSizeNoIncremental();
  
}

final class StrategySetSizeNoIncremental extends StrategySetSize {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategySetSize.java,v $ $revision$ $Date: 1998/11/02 15:55:08 $ Copyright 1998 INRIA.";

    public StrategySetSizeNoIncremental(ContextInterface aContext) {
	super(aContext);
    }
    public StrategySetSizeNoIncremental() {
	this(null);
    }
  
    public final void doAlgorithm(GlyphInterface aGlyph,
				  Dimension aSize) {
	aGlyph.setSize(aSize);
    }

    public final StrategySetSize clone(ContextInterface aContext) {
	return new StrategySetSizeNoIncremental(aContext);
    }
  
}

final class StrategySetSizeIncremental extends StrategySetSize {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/StrategySetSize.java,v $ $revision$ $Date: 1998/11/02 15:55:08 $ Copyright 1998 INRIA.";

    public StrategySetSizeIncremental(ContextInterface aContext) {
	super(aContext);
    }
    public StrategySetSizeIncremental() {
	this(null);
    }

    public final void doAlgorithm(GlyphInterface aGlyph,
				  Dimension aSize) {
	aGlyph.resetSize(aSize,getContext());
    }

    public final StrategySetSize clone(ContextInterface aContext) {
	return new StrategySetSizeIncremental(aContext);
    }
  
}

