//*****************************************************************************
//   Copyright (c) 1998 Bernard Sufrin, Oxford.
//   http://www.comlab.ox.ac.uk/oucl/people/bernard.sufrin.html
//*****************************************************************************

package figue.jape;

import  java.awt.image.*;
import  java.awt.*;

/**
     A character of size <TT> width*height</TT> represented as an image
     of size <TT>(width-xoffset)*(height-yoffset)</TT>.
     
     @see BitMapChar#BitMapChar
*/
public class ImageChar {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/jape/ImageChar.java,v $ $revision$ $Date: 1998/11/02 15:55:23 $ Copyright 1998, Bernard Sufrin, Oxford.";

    public int    width;
    public int    height;
    public int    xoffset;
    public int    yoffset;   
   
    private Image         image;
    private ColorModelGen cmg;
    private Component     c;
    private BitMapChar    ch;
   
    public ImageChar(int width, int height, int xoffset, int yoffset, Image image)
    { this.width   = width;
    this.height  = height;
    this.xoffset = xoffset;
    this.yoffset = yoffset;
    this.image   = image;
    }
   
    /** Create a glyph for the given Component using the specified colour model
	and bitmap character representation.
    */
    public ImageChar(Component c, BitMapChar ch, ColorModelGen cmg)
    {  this(ch.width, ch.height, ch.xoffset, ch.yoffset, null);
    this.cmg = cmg;
    this.c = c;
    this.ch = ch;
    }
   
    public final Image image()
    { if (image==null)
	{
	    MemoryImageSource mem = new MemoryImageSource(width, height, cmg.getColorModel(), ch.bitmap(), 0, width);              
	    this.image = c.createImage(mem);
	    // Release references to external resources once the image is built
	    ch = null;
	    c = null;
	    cmg = null;
	}
    return image;
    }
   
   
   
}

///+++ 
// interface ColorModelGen { java.awt.image.ColorModel getColorModel(); }







