//*****************************************************************************
//   Copyright (c) 1998 Bernard Sufrin, Oxford.
//   http://www.comlab.ox.ac.uk/oucl/people/bernard.sufrin.html
//*****************************************************************************

package figue.jape;

import  figue.jape.Font;
import  java.net.*;
import  java.awt.Color;
import  java.awt.Component;

/**
       A typeface implemented using the ImageFont representation
       and derived from a Tex PK file.
*/
public class PKFace implements TypeFace {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/jape/PKFace.java,v $ $revision$ $Date: 1998/11/02 15:55:24 $ Copyright 1998, Bernard Sufrin, Oxford.";

    public static PKFace open(Component component, URL fontbase, String name, int dpi, double scale)
    { return open(component, fontbase, name, dpi, scale, 8);
    }
    
    public static PKFace open(Component component, URL fontbase, String name, int dpi, double scale, int bits)
    { PKFace result = null;
    try
	{ result = new PKFace(component, fontbase, name, dpi, scale, bits);
	}
    catch (Exception e)
	{ //LOG.Error("PKFace.open: "+e);
	    e.printStackTrace();
	}
    finally 
	{ return result; }
    }
    
    private PKFace(Component component, URL fontbase, String name, int dpi, double scale, int bits)
	throws java.lang.Exception
    { this(component, fontbase, name, dpi, (int) ((double) dpi / ((double) component.getToolkit().getScreenResolution() * scale)), bits);
    }
    
    private PKFace(Component component, URL fontbase, String name, int dpi, int scale, int bits)
	throws java.lang.Exception
    { try
	{ base           = PKBitMapFont.openScaled(fontbase, name, dpi, scale).quantize(bits);
        this.component = component;
        this.scale     = scale;
        this.name      = name;
        this.bits      = bits;
	}
    catch (java.lang.Exception e) { throw e; }
    }
    
    /** Cut a new Font from this typeface */
    public Font getFont(Color fg, Color bg)
    {  return ImageFont.open(component, base, scale, bits, ColorModel.from(fg, bg));
    }
    
    public Font getFont(ColorScheme cs)
    {  return ImageFont.open(component, base, scale, bits, cs);
    }
    
    public Font getFont()
    {  return ImageFont.open(component, base, scale, bits, ColorModel.from(component));
    }
                
    private String     name      = null;
    private BitMapFont base      = null;
    private Component  component = null;
    private int        scale     = 1;
    private int        bits      = 8; // bits in the colour model
    
    public String toString()
    { return "PKFace("+component+","+name+","+","+scale+")";
}

}








