//*****************************************************************************
//   Copyright (c) 1998 Bernard Sufrin, Oxford.
//   http://www.comlab.ox.ac.uk/oucl/people/bernard.sufrin.html
//*****************************************************************************

package figue.jape;

/**
         A straightforward implementation of a BitMapFont. Constructed
         from a collection of bitmap characters, with vertical
         metrics calculated from the character data.
         <P>
         Lazy scaling of individual characters means that the original
         unscaled representation of a font may be retained longer than
         strictly necessary. 
*/
public class SimpleBitMapFont implements BitMapFont, java.io.Serializable { 

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/jape/SimpleBitMapFont.java,v $ $revision$ $Date: 1998/11/02 15:55:24 $ Copyright 1998, Bernard Sufrin, Oxford.";

    protected String       name;
    private   int          dpi, scalefactor=1;
    private   BitMapChar[] bitmaps;
  
    public  int          dpi()          { return dpi; }
    public  BitMapChar   getChar(int i) { return bitmaps[i]; }
    public  String       name()         { return name; }
  
    public String toString() { return name+"@"+scalefactor; }
  
    public SimpleBitMapFont(String name, int dpi, BitMapChar[] bitmaps)
    { this.name    = name;
    this.dpi     = dpi;
    setBitMaps(bitmaps);
    }
  
    public SimpleBitMapFont(String name, int dpi, BitMapChar[] bitmaps, int factor)
    { this.name    = name;
    this.dpi     = dpi;
    this.scalefactor = factor;
    setBitMaps(bitmaps);
    }
  
    public SimpleBitMapFont(String name, int dpi)
    { this.name    = name;
    this.dpi     = dpi;
    this.bitmaps = null;
    }
  
    public void setBitMaps(BitMapChar[] bitmaps)
    { this.bitmaps = bitmaps;
    //
    // Approximate the vertical metrics 
    //
    for (int i=0; i<256; i++) 
	if (this.bitmaps[i] !=null)
	    { BitMapChar ch = this.bitmaps[i]; 
            if (ch.height> maxheight) 
                maxheight = ch.height;
            if (ch.height-ch.yoffset > maxdescent) 
                maxdescent = ch.height-ch.yoffset; 
	    }
    }
  
    /** Make a scaled version of the font. Scaling of glyphs is lazy.
	@see BitMapChar#scale
    */
    public BitMapFont scale(int factor)
	throws java.lang.IllegalArgumentException
    { if (factor==1) return this;
    if (factor<1 || factor>16) 
	throw new java.lang.IllegalArgumentException("SimpleBitMapFont.scale: factor out of range 1..16: "+factor);
    BitMapChar[] bitmaps = new BitMapChar[256];
    for (int i=0; i<256; i++) 
        if (this.bitmaps[i] !=null)
	    bitmaps[i] = this.bitmaps[i].scale(factor); 
    return new SimpleBitMapFont(name, dpi / factor, bitmaps, factor);
    }
  
    /** Make a quantized version of the font. QUantization of glyphs is lazy.
	@see BitMapChar#quantize
    */
    public BitMapFont quantize(int bits)
    { if (bits==8) return this;
    BitMapChar[] bitmaps = new BitMapChar[256];
    for (int i=0; i<256; i++) 
        if (this.bitmaps[i] !=null)
	    bitmaps[i] = this.bitmaps[i].quantize(bits); 
    return new SimpleBitMapFont(name, dpi, bitmaps);
    }
  
    private int maxheight =0, 
	maxdescent=0;
   
    public int getHeight()  { return maxheight; }
    public int getAscent()  { return maxheight - maxdescent; }
    public int getDescent() { return maxdescent; }

}






