//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.multithread;

import figue.internal.Globals;
import figue.FacadeInterface;

public final class NotifyingThreadState implements NotifyingThreadStateInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/multithread/NotifyingThreadState.java,v $ $revision$ $Date: 1998/11/02 15:55:26 $ Copyright 1998 INRIA.";

    public NotifyingThreadState(FacadeInterface aFacade) {
	_facade = aFacade;
	_state = ThreadStateInterface.INIT;
    }
    public final void setState(ThreadStateInterface aState) {
	boolean theInactiveFlag; /// +++ NEW TRY BUG TEMPO
	synchronized ( this ) {
	    theInactiveFlag = ( aState == ThreadState.INACTIVE );
	    if ( Globals.DEBUG_MULTI_THREAD_OVERVIEW || Globals.DEBUG_MULTI_THREAD ) {
		System.out.println("-> setState " + aState);
	    }
	    _state = aState;
	}
	if ( theInactiveFlag ) { /// +++ NEW TRY BUG TEMPO
	    _facade.awake();/// +++ NEW TRY BUG TEMPO
	}/// +++ NEW TRY BUG TEMPO
	else {/// +++ NEW TRY BUG TEMPO
	    _facade.awakeWithoutInactive();/// +++ NEW TRY BUG TEMPO
	}/// +++ NEW TRY BUG TEMPO
    }
    public final void changeState(ThreadStateInterface aState) {
	synchronized ( this ) {
	    if ( Globals.DEBUG_MULTI_THREAD_OVERVIEW || Globals.DEBUG_MULTI_THREAD ) {
		System.out.println("-> changeState " + aState);
	    }
	    _state = aState;
	}
    }
//     protected final void putState(ThreadStateInterface aState) {
// 	if ( Globals.DEBUG_MULTI_THREAD_OVERVIEW || Globals.DEBUG_MULTI_THREAD ) {
// 	    System.out.println("-> putState " + aState);
// 	}
// 	_state = aState;
//     }
    public final void reportInvalidState() {
	_facade.reportError(InvalidStateError.INSTANCE);
    }
    public final synchronized boolean isDone() {
	return _state.isDone();
    }
    public final synchronized boolean isInactive() throws StoppedFacadeException {
	return _state.isInactive();
    }
    public final synchronized boolean isFormattable() throws StoppedFacadeException {
	return _state.isFormattable();
    }
    public final synchronized boolean isFormatting() throws StoppedFacadeException {
	return _state.isFormatting();
    }
    public final void startFormat() {
	_state.startFormat(this);
    }
    public final void endFormat() {
	_state.endFormat(this);
    }
    public final void suspendFormat() {
	_state.suspendFormat(this);
    }
    public final synchronized boolean isDrawable() throws StoppedFacadeException {
	return _state.isDrawable();
    }
    public final synchronized boolean isDrawing() throws StoppedFacadeException {
	return _state.isDrawing();
    }
    public final void startDraw() {
	_state.startDraw(this);
    }
    public final void endDraw() {
	_state.endDraw(this);
    }
    public final synchronized boolean isUpdatingGlyph() throws StoppedFacadeException {
	return _state.isUpdatingGlyph();
    }
    public final void startUpdateGlyph() {
	_state.startUpdateGlyph(this);
    }
    public final void endUpdateGlyph() {
	_state.endUpdateGlyph(this);
    }
    public final synchronized boolean isBuilding() throws StoppedFacadeException {
	return _state.isBuilding();
    }
    public final synchronized boolean isBuildingPhase() throws StoppedFacadeException {
	return _state.isBuildingPhase();
    }
    public final void startBuild() {
	_state.startBuild(this);
    }
    public final void endBuild() {
	_state.endBuild(this);
    }
    public final void quit() {
	_state.quit(this);
    }
    public final void init() {
	_state.init(this);
    }
    public final void fatalError() {
	_state.fatalError(this);
    }
    public final void dispose() {
	_facade = null;
	_state = null;
    }
    public final synchronized String toString() {
	StringBuffer theResult = new StringBuffer("NotifyingState<");
	theResult.append(_state.toString()).append(">");
	return theResult.toString();
    }
    private FacadeInterface _facade;
    private ThreadStateInterface _state;
}
