//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.path;

import figue.selection.SelectionInterface;
import figue.internal.PathVisitorInterface;

import java.io.IOException;
import java.util.Vector;

/**
 * [PENDING: update the comments] SelectionPath.
 */

public class SelectionPath implements SelectionPathInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/path/SelectionPath.java,v $ $revision$ $Date: 1998/11/02 15:55:32 $ Copyright 1998 INRIA.";
  
    public SelectionPath(int aRank) {
	super();
	_parent = null;
	_rank = aRank;
	_selections = new Vector(0,1);
	_children = new Vector(1,1);
    }
  
    public final int getRank() { 
	return _rank; 
    }

    public final SelectionInterface getSelection(int aRank) {
	return (SelectionInterface)_selections.elementAt(aRank);
    }

    public final void addSelection(SelectionInterface aSelection) {
	if ( ! _selections.contains(aSelection) ) {
	    internalAddSelection(aSelection);
	}
    }

    private final void internalAddSelection(SelectionInterface aSelection) {
	final int theSize = _selections.size();
	if ( theSize == 0 ) {
	    _selections.addElement(aSelection);
	    return;
	}
	final int thePriority = aSelection.getPriority();
	SelectionInterface theCurrentSelection = null;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSelection = getSelection(theRank);
	    if ( thePriority >= theCurrentSelection.getPriority() ) {
		_selections.insertElementAt(aSelection,theRank);
		return;
	    }
	}
	_selections.insertElementAt(aSelection,theSize);
    }

    public void removeAllSelections(PathVisitorInterface aVisitor) {
	removeSelections();
	aVisitor.addEmptyLeaf(this);
    }

    public void removeSelections() {
	_selections.removeAllElements();
	_children.removeAllElements();
    }

    public final void removeSelection(SelectionInterface aSelection,
				      PathVisitorInterface aVisitor) {
	_selections.removeElement(aSelection);
	if ( _selections.isEmpty() && _children.isEmpty() ) {
	    aVisitor.addEmptyLeaf(this);
	}
    }

    public final void inverseSelection(SelectionInterface aSelection,
				       PathVisitorInterface aVisitor) {
	if ( ! _selections.contains(aSelection) ) {
	    internalAddSelection(aSelection);
	}
	else {
	    removeSelection(aSelection,aVisitor);
	}
    }
  
    public final int numberOfSelections() {
	return _selections.size();
    }

    public final SelectionPathInterface getParent() {
	return _parent;
    }

    public final void setParent(SelectionPathInterface aSelectionPath) {
	_parent = aSelectionPath;
    }

    public final void addChild(SelectionPathInterface aChild) {
	final int theChildRank = aChild.getRank();
	aChild.setParent(this);
	final int theOldSize = _children.size();
	int theRank = 0;
	while ( theRank < theOldSize ) {
	    final int theCurrentRank = getChild(theRank).getRank();
	    if ( theCurrentRank == theChildRank ) {
		SelectionPathInterface theOldChild = getChild(theRank);
		final int theSize = aChild.numberOfSelections();
		for( int i = 0; i < theSize; i++ ) {
		    theOldChild.addSelection(aChild.getSelection(i));
		}
		final int theOtherSize = aChild.numberOfChildren();
		for ( int j = 0; j < theOtherSize; j++ ) {
		    theOldChild.addChild(aChild.getChild(j));
		}
		return;
	    }
	    if ( theChildRank < theCurrentRank ) {
		break;
	    }
	    theRank++;
	}
	_children.insertElementAt(aChild,theRank);
    }
  
    public final SelectionPathInterface getChild(int aRank) {
	return (SelectionPathInterface)_children.elementAt(aRank);
    }

    public final int indexOf(int aRank) {
	final int theSize = _children.size();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    final int theLogicalRank = 
		((SelectionPathInterface)_children.elementAt(theRank)).getRank();
	    if ( theLogicalRank == aRank ) {
		return theRank;
	    }
	    if ( theLogicalRank > aRank ) {
		return -1;
	    }
	}
	return -1;
    }

    public final SelectionPathInterface search(int aRank) {
	final int theSize = _children.size();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    final SelectionPathInterface theChild = (SelectionPathInterface)_children.elementAt(theRank);
	    final int theLogicalRank = theChild.getRank();
	    if ( theLogicalRank == aRank ) {
		return theChild;
	    }
	    if ( theLogicalRank > aRank ) {
		return null;
	    }
	}
	return null;
    }

    public final void removeChild(SelectionPathInterface aChild) {
	_children.removeElement(aChild);
    }

    public final int numberOfChildren() {
	return _children.size();
    }

    public final void cutEmptyLeaf() {
	SelectionPathInterface theNode = this;
	while ( ( theNode.numberOfSelections() == 0 ) &&
		( theNode.numberOfChildren() == 0 ) && 
		( theNode.getParent() != null ) ) {
	    SelectionPathInterface theOldNode = theNode;
	    theNode = theNode.getParent();
	    theNode.removeChild(theOldNode);
	}
    }

    public final boolean contains(SelectionInterface aSelection) {
	return _selections.contains(aSelection);
    }

    public String toString() {
	StringBuffer theResult = new StringBuffer("SelectionPath<\n");
	internalToString(theResult,"");
	theResult.append(">");
	return theResult.toString();
    }

    public final void internalToString(StringBuffer aBuffer, String aPath) {
	aBuffer.append("  - ").append(aPath).append(".").append(_rank)
	    .append(": {");
	final int theSize = _selections.size();
	boolean theFirstFlag = true;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    if ( theFirstFlag ) {
		theFirstFlag = false;
	    }
	    else {
		aBuffer.append(", ");
	    }
	    aBuffer.append(_selections.elementAt(theRank).toString());
	}
	aBuffer.append("}\n");
	final String thePath = aPath + "." + String.valueOf(getRank());
	final int theOtherSize = _children.size();
	for( int theRank = 0; theRank < theOtherSize; theRank++ ) {
	    ((SelectionPathInterface)_children.elementAt(theRank)).
		internalToString(aBuffer,thePath);
	}
    }

    private SelectionPathInterface _parent;
    private int _rank;
    private Vector _selections;
    private Vector _children;
  
}

