//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.resource;

import figue.error.ShouldNotBeCalledError;

import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] Font.
 * @level advanced
 */

public abstract class Font implements FontInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/Font.java,v $ $revision$ $Date: 1998/11/02 15:55:37 $ Copyright 1998 INRIA.";

    protected Font(String aMnemonic,
		   String aKey,
		   FontFamily aFamily,
		   FontStyle aStyle,
		   int aPointSize,
		   boolean anUnderlinedFlag) {
	_name = aMnemonic;
	_key = aKey;
	_family = aFamily;
	_style = aStyle;
	_size = aPointSize;
	_underlinedFlag = anUnderlinedFlag;
    }

    public final String getName() {
	return _name;
    }

    public final String getKey() { 
	return _key; 
    }
  
    public final FontFamily getFamily() {
	return _family;
    }
  
    public final FontStyle getStyle() {
	return _style;
    }
  
    public final int getSize() {
	return _size; 
    }

    public final boolean isUnderlined() {
	return _underlinedFlag;
    }

    public java.awt.Font getAwtFont() {
	throw new ShouldNotBeCalledError();
    }

    public final String toString() {
	final StringBuffer theResult = new StringBuffer("Font<");
	theResult.append(getName()).append(':').append(_family)
	    .append(_size).append(_style).append('>');
	return theResult.toString();
    }

    public final void writeState(Writer anOutput) throws IOException {
	anOutput.write("<Font Name=\"");
	Utility.writeStringAsXmlValue(anOutput,_name);
	anOutput.write("\" Family=\"");
	Utility.writeStringAsXmlValue(anOutput,_family.getName());
	anOutput.write("\" Style=\"");
	Utility.writeStringAsXmlValue(anOutput,_style.getName());
	anOutput.write("\" Size=\"");
	anOutput.write(Integer.toString(_size));
	anOutput.write("\" Underlined=\"");
	if ( _underlinedFlag ) {
	    anOutput.write("true");
	}
	else {
	    anOutput.write("false");
	}
	anOutput.write("\"/>");
    }

    private String _name;
    private String _key;
    private FontFamily _family;
    private FontStyle _style;
    private int _size;
    private boolean _underlinedFlag;
  
}

