//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.resource;

import figue.error.ErrorHandler;
import figue.error.ErrorHandlerInterface;
import figue.error.ImplementationError;
import figue.error.ShouldNotBeCalledError;
import figue.Constants;
import figue.error.Warning;
import figue.error.Error;

/**
 * [PENDING: update the comments] ResourceInitializer.
 * @level advanced
 */

public abstract class ResourceInitializer extends ErrorHandler implements ResourceInitializerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/ResourceInitializer.java,v $ $revision$ $Date: 1998/11/02 15:55:41 $ Copyright 1998 INRIA.";

    public ResourceInitializer() {
    }

    protected final FontInterface newFont(ResourceOwnerInterface anOwner,
					  String aName) {
	try {
	    return anOwner.getFacade().getFont(aName);
	}
	catch ( WrongResourceException anException ) {
	    if ( aName.equals(Constants.DEFAULT_FONT) ) {
		return anOwner.getFacade().doGetDefaultFont();
	    }
	    else {
		anOwner.reportError(new Warning("Font not found : " + aName + " => use default value instead."));
		final FontInterface theDefaultFont = anOwner.getFacade().doGetDefaultFont();
		try {
		    return anOwner.getFacade().doGetFont(aName,theDefaultFont.getFamily(),theDefaultFont.getStyle(),theDefaultFont.getSize(),theDefaultFont.isUnderlined());
		}
		catch ( WrongResourceException anotherException ) {
		    anotherException.printStackTrace();
		    throw new ImplementationError("Can't allocate font " + aName);
		}
	    }
	}
    }
  
    protected final ColorInterface newColor(ResourceOwnerInterface anOwner,
					    String aName) {
	try {
	    return anOwner.getFacade().getColor(aName);
	}
	catch ( WrongResourceException anException ) {
	    if ( aName.equals(Constants.DEFAULT_COLOR) ) {
		return anOwner.getFacade().doGetDefaultColor();
	    }
	    else {
		anOwner.reportError(new Warning("Color not found : " + aName + " => use default value instead."));
		final ColorInterface theDefaultColor = anOwner.getFacade().doGetDefaultColor();
		try {
		    return anOwner.getFacade().doGetColor(aName,theDefaultColor.getRed(),theDefaultColor.getGreen(),theDefaultColor.getBlue());
		}
		catch ( WrongResourceException anotherException ) {
		    throw new ImplementationError("Can't allocate color " + aName);
		}
	    }
	}
    }

    protected final ColorInterface newBackground(ResourceOwnerInterface anOwner,
						 String aName) {
	try {
	    return anOwner.getFacade().getColor(aName);
	}
	catch ( WrongResourceException anException ) {
	    if ( aName.equals(Constants.DEFAULT_BACKGROUND) ) {
		return anOwner.getFacade().doGetDefaultBackground();
	    }
	    else {
		anOwner.reportError(new Warning("Color not found : " + aName + " => use default value instead."));
		final ColorInterface theDefaultBackground = anOwner.getFacade().doGetDefaultBackground();
		try {
		    return anOwner.getFacade().doGetColor(aName,theDefaultBackground.getRed(),theDefaultBackground.getGreen(),theDefaultBackground.getBlue());
		}
		catch ( WrongResourceException anotherException ) {
		    throw new ImplementationError("Can't allocate color " + aName);
		}
	    }
	}
    }
  
    protected final StyleInterface newStyle(ResourceOwnerInterface anOwner,
					    String aName) {
	try {
	    return anOwner.getFacade().getStyle(aName);
	}
	catch ( WrongResourceException anException ) {
	    if ( aName.equals(Constants.DEFAULT_STYLE) ) {
		return anOwner.getFacade().doGetDefaultStyle();
	    }
	    else {
		anOwner.reportError(new Warning("Style not found : " + aName + " => use default value instead."));
		final StyleInterface theDefaultStyle = anOwner.getFacade().doGetDefaultStyle();
		try {
		    return anOwner.getFacade().doGetStyle(aName,theDefaultStyle.getFont(),theDefaultStyle.getColor(),theDefaultStyle.getBackground());
		}
		catch ( WrongResourceException anotherException ) {
		    throw new ImplementationError("Can't allocate style " + aName);
		}
	    }
	}
    }

    public final boolean hasNextErrorHandler() {
	return true;
    }
  
    public final ErrorHandlerInterface getNextErrorHandler() {
	return getOwner();
    }

    public final void manageFatalError(Error anError) {
	throw new ShouldNotBeCalledError();
    }

    public final void manageWarning(Warning aWarning) {
	throw new ShouldNotBeCalledError();
    }

    public final boolean canHandleError() {
	return false;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("ResourceInitializer<");
	theResult.append(getName()).append(">");
	return theResult.toString();
    }

}
