//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.selection; 

import figue.box.GlyphInterface;
import figue.geometry.Transformer;
import figue.geometry.Dimension;
import figue.path.SelectionPathInterface;
import figue.path.SelectionPathInterface;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;

/**
 * [PENDING: update the comments] DefaultHighlighter.
 */

public class DefaultHighlighter implements HighlighterInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/selection/DefaultHighlighter.java,v $ $revision$ $Date: 1998/11/02 15:55:44 $ Copyright 1998 INRIA.";
    
    protected DefaultHighlighter() {
	super();
    }

    public final ContextMementoInterface draw2D(GlyphInterface aGlyph,
						ContextInterface aContext,
						Transformer aTransformer,
						SelectionPathInterface aSelectionPath) {
	final ContextMementoInterface theMemento = computeSelectionMemento(aContext,
									   aSelectionPath);
	if ( ( theMemento != null ) && 
	     ( theMemento.getBackground() != null) ) {
	    highlightArea(aGlyph,aGlyph.getSizeEstimation(),aTransformer,
			  aContext,theMemento);
	}
	return theMemento;
    }
    
    public final ContextMementoInterface draw2DClean(GlyphInterface aGlyph,
						     ContextInterface aContext,
						     Transformer aTransformer,
						     SelectionPathInterface aSelectionPath) {
	final ContextMementoInterface theMemento = computeSelectionMemento(aContext,
									   aSelectionPath);
	if ( ( theMemento != null ) && 
	     ( theMemento.getBackground() != null) ) {
	    highlightArea(aGlyph,aGlyph.getSizeValue(),aTransformer,
			  aContext,theMemento);
	}
	return theMemento;
    }

    protected void highlightArea(GlyphInterface aGlyph,
				 Dimension aSize,
				 Transformer aTransformer,
				 ContextInterface aContext,
				 ContextMementoInterface aMemento) {
	aContext.fillRectangle(
			       aTransformer.applyToX(aGlyph.getOriginX()) - aSize.getUpperLeftX(),
			       aTransformer.applyToY(aGlyph.getOriginY()) - aSize.getUpperLeftY(),
			       aSize.getWidth(),
			       aSize.getHeight());
    }
    
    private final ContextMementoInterface computeSelectionMemento(ContextInterface aContext,
								  SelectionPathInterface aSelectionPath) {
	ContextMementoInterface theMemento = null;
	SelectionInterface theSelection;
	if ( aSelectionPath != null ) {
	    final int theSelectionSize = aSelectionPath.numberOfSelections();
	    for( int theRank = 0; theRank < theSelectionSize; theRank++ ) {
		theSelection = aSelectionPath.getSelection(theRank);
		if ( theSelection.isVisible() ) {
		    if ( theMemento == null ) {
			theMemento = aContext.set(theSelection);
		    }
		    else {
			aContext.set(theSelection,theMemento);
		    }
		    if ( theMemento.isFull() ) {
			break;
		    }
		}
	    }
	}
	return theMemento;
    }
    
    public final static DefaultHighlighter INSTANCE = new DefaultHighlighter();
    
}


