//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.utility;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Event;
import java.awt.Rectangle;
import java.applet.Applet;
import java.net.URL;
import java.awt.Frame;
import java.awt.Component;
import java.awt.Frame;

public final class WrongVersion extends Canvas {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/utility/WrongVersion.java,v $ $revision$ $Date: 1998/11/02 15:56:03 $ Copyright 1998 INRIA.";

    public WrongVersion(Applet anApplet,
			String aMinVersion,
			URL aRedirectPage,
			URL anHelpPage,
			Color aColor,
			Font aFont) {
	super();
	_frame = null;
	_applet = anApplet;
	setBackground(aColor);
	setFont(aFont);
	_firstLine = "You must use ";
	_firstLineLink = aMinVersion;
	_secondLine = "to see the Applets.";
	_thirdLineLink = "Want to see";
	_thirdLine = " what you are missing ?";
	FontMetrics theMetrics = getFontMetrics(getFont());
	_offset = theMetrics.getAscent();
	_lineHeight = theMetrics.getHeight();
	_height = _lineHeight * 3;
	_firstLineWidth = theMetrics.stringWidth(_firstLine);
	_firstLineLinkWidth = theMetrics.stringWidth(_firstLineLink);
	_thirdLineLinkWidth = theMetrics.stringWidth(_thirdLineLink);
	_firstLineLinkArea = new Rectangle(_firstLineWidth,0,_firstLineWidth + _firstLineLinkWidth,_lineHeight);
	_thirdLineLinkArea = new Rectangle(0,_lineHeight * 2,_thirdLineLinkWidth,_lineHeight);
	_width = Math.max(Math.max(theMetrics.stringWidth(_firstLine + _firstLineLink),
				   theMetrics.stringWidth(_secondLine)),
			  theMetrics.stringWidth(_thirdLineLink + _thirdLine));
	_clickingHelpFlag = false;
	_clickingSampleFlag = false;
	_samplePage = aRedirectPage;
	_helpPage = anHelpPage;
	show();
    }

    public final void paint(Graphics g) {
	g.setColor(Color.black);
	g.drawString(_firstLine,0,_offset);
	if ( _clickingHelpFlag ) {
	    g.setColor(Color.red);
	}
	else {
	    g.setColor(Color.blue);
	}
	g.drawString(_firstLineLink,_firstLineWidth,_offset);
	g.drawLine(_firstLineWidth,_offset + 1,_firstLineWidth + _firstLineLinkWidth,_offset + 1);
	g.setColor(Color.black);
	g.drawString(_secondLine,0,_offset + _lineHeight);
	if ( _clickingSampleFlag ) {
	    g.setColor(Color.red);
	}
	else {
	    g.setColor(Color.blue);
	}
	g.drawString(_thirdLineLink,0,_offset + (_lineHeight * 2));
	g.drawLine(0,_offset + 1 + (_lineHeight * 2),_thirdLineLinkWidth,_offset + 1 + (_lineHeight * 2));
	g.setColor(Color.black);
	g.drawString(_thirdLine,_thirdLineLinkWidth,_offset + (_lineHeight * 2));
    }

    public final void update(Graphics g) {
	paint(g);
    }

    public final Dimension preferredSize() {
	return minimumSize();
    }
    
    public final Dimension minimumSize() {
	return new Dimension(_width,_height);
    }
    
    public final Dimension getMinimumSize() {
	return minimumSize();
    }
    
    public final Dimension getPreferredSize() {
	return preferredSize();
    }

    public final boolean mouseMove(Event anEvent, int x, int y) {
	if ( overHelp(anEvent,x,y) ) {
	    changeCursor(Frame.HAND_CURSOR);
	    return true;
	}
	else {
	    if ( overSample(anEvent,x,y) ) {
		changeCursor(Frame.HAND_CURSOR);
		return true;
	    }
	    else {
		changeCursor(Frame.DEFAULT_CURSOR);
	    }
	}
        return false;
    }

    public final boolean mouseExit(Event anEvent, int x, int y) {
	changeCursor(Frame.DEFAULT_CURSOR);
	return false;
    }

    private final void changeCursor(int aCursor) {
	if ( _frame == null ) {
	    Component theComponent = this;
	    while ( theComponent.getParent() != null ) {
		theComponent = theComponent.getParent();
	    }
	    try {
		_frame = (Frame)theComponent;
	    }
	    catch ( ClassCastException anException ) {
		;
	    }
	}
	if ( _frame != null ) {
	    _frame.setCursor(aCursor);
	}
    }

    public final boolean mouseDown(Event anEvent,int x,int y) {
	if ( overHelp(anEvent,x,y) ) {
	    _clickingHelpFlag = true;
	    repaint(0);
	    return true;
	}
	else {
	    if ( overSample(anEvent,x,y) ) {
		_clickingSampleFlag = true;
		repaint(0);
		return true;
	    }
	}
	return false;
    }

    public final boolean mouseUp(Event anEvent,int x,int y) {
	if ( overHelp(anEvent,x,y) && _clickingHelpFlag ) {
	    _clickingHelpFlag = false;
	    repaint(0);
	    openHelp();
	    return true;
	}
	else {
	    if ( overSample(anEvent,x,y) && _clickingSampleFlag ) {
		_clickingSampleFlag = false;
		repaint(0);
		openSample();
		return true;
	    }
	    else {
		if ( _clickingHelpFlag || _clickingSampleFlag ) {
		    changeCursor(Frame.DEFAULT_CURSOR);
		    _clickingHelpFlag = false;
		    _clickingSampleFlag = false;
		    repaint(0);
		}
	    }
	}
	return false;
    }

    private final boolean overHelp(Event anEvent, int x, int y) {
	return _firstLineLinkArea.contains(x,y);
    }
    
    private final boolean overSample(Event anEvent, int x, int y) {
	return _thirdLineLinkArea.contains(x,y);
    }

    private final void openHelp() {
	if ( _helpPage != null ) {
	    _applet.getAppletContext().showDocument(_helpPage);
	    _applet.destroy();
	}
    }

    private final void openSample() {
	if ( _samplePage != null ) {
	    _applet.getAppletContext().showDocument(_samplePage);
	    _applet.destroy();
	}
    }

    public final void dispose() {
	_firstLine = null;
	_firstLineLink = null;
	_secondLine = null;
	_thirdLineLink = null;
	_thirdLine = null;
	_firstLineLinkArea = null;
	_thirdLineLinkArea = null;
	_samplePage = null;
	_helpPage = null;
	_applet = null;
	_frame = null;
	hide();
    }

    private String _firstLine;
    private String _firstLineLink;
    private String _secondLine;
    private String _thirdLineLink;
    private String _thirdLine;
    private int _width;
    private int _height;
    private int _lineHeight;
    private int _offset;
    private int _firstLineWidth;
    private int _firstLineLinkWidth;
    private int _thirdLineLinkWidth;
    private Rectangle _firstLineLinkArea;
    private Rectangle _thirdLineLinkArea;
    private boolean _clickingHelpFlag;
    private boolean _clickingSampleFlag;
    private URL _samplePage;
    private URL _helpPage;
    private Applet _applet;
    private Frame _frame;

}
